/*! 
 * \file jfet_init.c
 *
 * \brief  initialization and startup code for JFET hardware and sensors
 *
 * \copyright  J Factor Embedded Technologies LLC. 2020
 * \copyright  TJM Embedded Software LLC. 2020
 * \author  T. J. Mulrooney
 */
 /*
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF J-Factor Embedded Technologies.
 *
*/
//#include <console/console.h>
#include <zephyr/kernel.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
//#include <device.h>
//#include <drivers/adc.h>
//#include <hal/nrf_saadc.h>
//#include <drivers/gpio.h>
#include <zephyr/drivers/pwm.h>
#include <dk_buttons_and_leds.h>
//#include <modem/modem_info.h>
#include "jfet_common.h"
#include "nextion/NexHardware.h"
#include "nextion/Nextion.h"
#include <math.h>
#include <date_time.h>
#include <zephyr/sys/reboot.h>
//#include "events/data_module_event.h"
//#include "events/app_module_event.h"
//#include "modules_common.h"
#include "mainpage.h"

#ifdef NO_UART0
#include "power_manager.h"
#endif


///* Module name is used by the event manager macros in this file */
//#define MODULE jfet_module    //!< Module name is used by the event manager macros in this file
//
//#include <logging/log.h>
//#include <logging/log_ctrl.h>
//
//LOG_MODULE_REGISTER(MODULE, CONFIG_APPLICATION_MODULE_LOG_LEVEL);
//
//
///* prototypes */
//static int jfet_dk_leds_init(void);
uint8_t checkTempAlarms(void);
void checkTempAlarmsValid(void);
uint8_t checkEZOAlarms(void);
void checkEZOAlarmsValid(void);
uint8_t checkHumidityAlarms(void);
void checkHumidityAlarmsValid(void);
uint8_t checkPressureAlarms(void);
void checkPressureAlarmsValid(void);
//uint8_t checkPowerAlarms(void);
//void checkPowerAlarmsValid(void);
void uart0_cb(const struct device *dev, void *user_data);
void uart2_cb(const struct device *dev, void *user_data);
//void uart_cb(const struct device *x, void *user_data);
void refreshDisplay();
uint8_t addSavedJSONPacket(char *buff);
int time_NTP_server_get(void);
//void getNTPTime(struct tm *pTime);
void setPowerMode(int mode);

/* globals */
uint8_t num_end_seen = 0;   //!< flag indicating if Nextion display has sent the end code
uint8_t num_chars_seen = 0; //!< number of characters sent from Nextion display
//struct k_work_q application_work_q;
uint8_t uart0_buf[80];     //!< buffer of charactres sent from UART0
uint8_t uart2_buf[100];     //!< buffer of charactres sent from Nextion display on UART2
uint8_t display_line_seen = false;  //!< line with end seen from Nextion display
//
///* locals */
//static uint8_t cliIdleWait = 0;   //!< count down timer to wait in idle state on reset
//
/* externals */
extern struct k_fifo fifo_uart_rx_data;
extern struct k_fifo fifo_uart2_rx_data;

/* start GPIO test */
#include <zephyr/drivers/gpio.h>

/* outputs */
#define SW2CTRL_DEV_NODE            DT_GPIO_CTLR(DT_ALIAS(sw2ctrl), gpios)
#define SW2CTRL_DEV_PIN             DT_GPIO_PIN(DT_ALIAS(sw2ctrl), gpios)
#define SW2CTRL_DEV_PIN_DTS_FLAGS   DT_GPIO_FLAGS(DT_ALIAS(sw2ctrl), gpios)

#define CS2N_DEV_NODE            DT_GPIO_CTLR(DT_ALIAS(spi_cs2n), gpios)
#define CS2N_DEV_PIN             DT_GPIO_PIN(DT_ALIAS(spi_cs2n), gpios)
#define CS2N_DEV_PIN_DTS_FLAGS   DT_GPIO_FLAGS(DT_ALIAS(spi_cs2n), gpios)

#define CS3N_DEV_NODE            DT_GPIO_CTLR(DT_ALIAS(spi_cs3n), gpios)
#define CS3N_DEV_PIN             DT_GPIO_PIN(DT_ALIAS(spi_cs3n), gpios)
#define CS3N_DEV_PIN_DTS_FLAGS   DT_GPIO_FLAGS(DT_ALIAS(spi_cs3n), gpios)

#define BAT_MON_EN_DEV_NODE            DT_GPIO_CTLR(DT_ALIAS(batmonen), gpios)
#define BAT_MON_EN_DEV_PIN             DT_GPIO_PIN(DT_ALIAS(batmonen), gpios)
#define BAT_MON_EN_DEV_PIN_DTS_FLAGS   DT_GPIO_FLAGS(DT_ALIAS(batmonen), gpios)

const struct device *sw2ctrl_dev = DEVICE_DT_GET(SW2CTRL_DEV_NODE);
const struct device *spi2_csn_dev = DEVICE_DT_GET(CS2N_DEV_NODE);
const struct device *spi3_csn_dev = DEVICE_DT_GET(CS3N_DEV_NODE);
const struct device *bat_mon_en_dev = DEVICE_DT_GET(BAT_MON_EN_DEV_NODE);

const struct pwm_dt_spec pwm_led0 = PWM_DT_SPEC_GET(DT_ALIAS(pwm_led0));

#define MIN_PERIOD PWM_MSEC(10U) / 128U
#define MAX_PERIOD PWM_MSEC(10U)

/* Inputs */
#define GPIO_09_DEV_NODE            DT_GPIO_CTLR(DT_ALIAS(gpio_09), gpios)
#define GPIO_09_DEV_PIN             DT_GPIO_PIN(DT_ALIAS(gpio_09), gpios)
#define GPIO_09_DEV_PIN_DTS_FLAGS   DT_GPIO_FLAGS(DT_ALIAS(gpio_09), gpios)

#define CHRG_DEV_NODE            DT_GPIO_CTLR(DT_ALIAS(chrg), gpios)
#define CHRG_DEV_PIN             DT_GPIO_PIN(DT_ALIAS(chrg), gpios)
#define CHRG_DEV_PIN_DTS_FLAGS   DT_GPIO_FLAGS(DT_ALIAS(chrg), gpios)

#define ACPR_DEV_NODE            DT_GPIO_CTLR(DT_ALIAS(acpr), gpios)
#define ACPR_DEV_PIN             DT_GPIO_PIN(DT_ALIAS(acpr), gpios)
#define ACPR_DEV_PIN_DTS_FLAGS   DT_GPIO_FLAGS(DT_ALIAS(acpr), gpios)

const struct device *gpio_09_dev = DEVICE_DT_GET(GPIO_09_DEV_NODE);
const struct device *chrg_dev = DEVICE_DT_GET(CHRG_DEV_NODE);
const struct device *acpr_dev = DEVICE_DT_GET(ACPR_DEV_NODE);

void gpio09Callback(struct device *dev, struct gpio_callback *cb, uint32_t pins)
{
	int test;

	test = gpio_pin_get(gpio_09_dev, GPIO_09_DEV_PIN);
	printf("GPIO_09 Interrupt %d\r\n",test);
}

void chrgCallback(struct device *dev, struct gpio_callback *cb, uint32_t pins)
{
	int test;

	test = gpio_pin_get(chrg_dev, CHRG_DEV_PIN);
	printf("CHRG Interrupt %d\r\n",test);
}

void acprCallback(struct device *dev, struct gpio_callback *cb, uint32_t pins)
{
	int test;

	test = gpio_pin_get(acpr_dev, ACPR_DEV_PIN);
	printf("ACPR Interrupt %d\r\n",test);
}

/* end GPIO test */

/* start ADC test */
#include <zephyr/drivers/adc.h>
#include <hal/nrf_saadc.h>
#include <math.h>

#define BUFFER_SIZE 2  //!< number of ADC channels to read

#if DT_NODE_HAS_STATUS(DT_ALIAS(adc), okay)
#define ADC_DEV_NODE DT_ALIAS(adc)
#elif
#error "Please set the correct ADC device"
#endif
//#define ADC_DEVICE_NAME DT_ADC_0_NAME
#define ADC_RESOLUTION 10
#define ADC_GAIN ADC_GAIN_1_6
#define ADC_REFERENCE ADC_REF_INTERNAL
#define ADC_ACQUISITION_TIME ADC_ACQ_TIME(ADC_ACQ_TIME_MICROSECONDS, 10)
#define ADC_1ST_CHANNEL_ID 0
#define ADC_1ST_CHANNEL_INPUT NRF_SAADC_INPUT_AIN0
#define ADC_2ND_CHANNEL_ID 1
#define ADC_2ND_CHANNEL_INPUT NRF_SAADC_INPUT_AIN1
#define ADC_3RD_CHANNEL_ID 2
#define ADC_3RD_CHANNEL_INPUT NRF_SAADC_INPUT_AIN2
#define ADC_4TH_CHANNEL_ID 3
#define ADC_4TH_CHANNEL_INPUT NRF_SAADC_INPUT_AIN3
#define ADC_5TH_CHANNEL_ID 4
#define ADC_5TH_CHANNEL_INPUT NRF_SAADC_INPUT_AIN4
#define ADC_6ST_CHANNEL_ID 5
#define ADC_6ST_CHANNEL_INPUT NRF_SAADC_INPUT_AIN5

const struct device *const adc_dev = DEVICE_DT_GET(ADC_DEV_NODE);
//float adc_voltage[2] = {0};         //!<last ADC voltage read [0] battery, [1] analog in

#if 1
static const struct adc_channel_cfg m_4th_channel_cfg =   //!< ADC config for ADC input
{
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_4TH_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_4TH_CHANNEL_INPUT,
#endif
};
#endif

#if 0
static const struct adc_channel_cfg m_3rd_channel_cfg = 
{
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_3RD_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_3RD_CHANNEL_INPUT,
#endif
};
#endif

#if 1
static const struct adc_channel_cfg m_2nd_channel_cfg =   //!< ADC config for Battery input
{
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_2ND_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_2ND_CHANNEL_INPUT,
#endif
};
#endif

#if 0
static const struct adc_channel_cfg m_1st_channel_cfg = 
{
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_1ST_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_1ST_CHANNEL_INPUT,
#endif
};
#endif

static int16_t m_sample_buffer[BUFFER_SIZE];  //!< buffer to receive ADC data from configured channels

/*!
 * \fn int adc_sample(void)
 *
 * \brief get a single reading from ADC channel
 *
 * will read ADC_4TH_CHANNEL_ID\n
 * resoultion set to ADC_RESOLUTION in millivolts\n
 * will printout floating point value based on 3.6V max\n
 *
 * \return ADC_RESOLUTION value on ADC_4TH_CHANNEL_ID if pass or -1 if failed\n
 */
int adc_sample(void)
{
	int ret;

	const struct adc_sequence sequence = 
    {
//		.channels = BIT(ADC_1ST_CHANNEL_ID) | BIT(ADC_2ND_CHANNEL_ID) | BIT(ADC_3RD_CHANNEL_ID) | BIT(ADC_4TH_CHANNEL_ID),
		.channels =  BIT(ADC_2ND_CHANNEL_ID) | BIT(ADC_4TH_CHANNEL_ID),
		.buffer = m_sample_buffer,
		.buffer_size = sizeof(m_sample_buffer),
		.resolution = ADC_RESOLUTION,
	};

	if (!adc_dev) 
    {
		return -1;
	}

//    jfet_dk_set_led(BATMONENB,1);
    k_sleep(K_MSEC(500));
// k_sleep(K_MSEC(1000));
    ret = adc_read(adc_dev, &sequence);
    if(ret !=0)
      printk("ADC read err: %d\n", ret);

         /* Print the AIN0 values */
	for (int i = 0; i < BUFFER_SIZE; i++) 
    {
//		adc_voltage = (float)(((float)m_sample_buffer[i] / 1023.0f) * 3600.0f);
//		printf("Measured voltage: %3.2f V\n", adc_voltage);
//        printf("den: %3.2f\r\n",((pow(2,ADC_RESOLUTION) -1 ) * 1.0)  * 3600.0f);
		adc_voltage[i] = (float)(((float)m_sample_buffer[i] / ((pow(2,ADC_RESOLUTION) -1 ) * 1.0) ) * 3600.0f);
//		printf("Measured voltage: %3.2f V\n", adc_voltage);
//        adc_voltage[i] = adc_voltage[i]/1000.0;
        adc_voltage[i] = adc_voltage[i]/10000.0;
//		printf("ADC raw value: %d 0x%03X\n", m_sample_buffer[i], m_sample_buffer[i]);
//		printf("Measured voltage: %3.2f V\n", adc_voltage[i] * 10);
	}
//    jfet_dk_set_led(BATMONENB,0);

	return ret;
}

/* end ADC test */

/* start SPI test */
#include <zephyr/drivers/spi.h>

#if DT_NODE_HAS_STATUS(DT_ALIAS(spi0), okay)
#define SPI_DEV_NODE	DT_ALIAS(spi0)
#elif DT_NODE_HAS_STATUS(DT_ALIAS(spi1), okay)
#define SPI_DEV_NODE	DT_ALIAS(spi1)
#elif DT_NODE_HAS_STATUS(DT_ALIAS(spi2), okay)
#define SPI_DEV_NODE	DT_ALIAS(SPIM2_EASYDMA_MAXCNT_SIZ3)
#elif DT_NODE_HAS_STATUS(DT_ALIAS(spi3), okay)
#define SPI_DEV_NODE	DT_ALIAS(spi3)
#else
#error "Please set the correct SPI device"
#endif
//#define SPI_DRV_NAME	CONFIG_SPI_LOOPBACK_DRV_NAME

const struct device *const spi_dev = DEVICE_DT_GET(SPI_DEV_NODE);

/* end SPI test */

#if 1
/* start I2C test */
#include <zephyr/drivers/i2c.h>

//#if DT_NODE_HAS_STATUS(DT_ALIAS(i2c0), okay)
//#define I2C_DEV_NODE	DT_ALIAS(i2c0)
#if DT_NODE_HAS_STATUS(DT_ALIAS(i2c1), okay)
#define I2C_DEV_NODE	DT_ALIAS(i2c1)
//#elif DT_NODE_HAS_STATUS(DT_ALIAS(i2c2), okay)
//#define I2C_DEV_NODE	DT_ALIAS(i2c2)
#//elif DT_NODE_HAS_STATUS(DT_ALIAS(i2c3), okay)
//#define I2C_DEV_NODE	DT_ALIAS(i2c3)
#else
#error "Please set the correct I2C device"
#endif

const struct device *const i2c_dev = DEVICE_DT_GET(I2C_DEV_NODE);
uint32_t i2c_cfg = I2C_SPEED_SET(I2C_SPEED_STANDARD) | I2C_MODE_CONTROLLER;
//uint32_t i2c_cfg = I2C_SPEED_SET(I2C_SPEED_FAST) | I2C_MODE_CONTROLLER;

/* end I2C test */
#endif

/* start UART test */

struct uart_data_t uartData;
struct uart_data_t uart2Data;

#if DT_NODE_HAS_STATUS(DT_ALIAS(uart0), okay)
#define UART0_DEV_NODE	DT_ALIAS(uart0)
#else
//#error "Please set the correct UART0 device"
#warning "UART0 not available"
#endif

#if DT_NODE_HAS_STATUS(DT_ALIAS(uart2), okay)
#define UART2_DEV_NODE	DT_ALIAS(uart2)
#else
#error "Please set the correct UART2 device"
#endif

const struct device *const uart0_dev = DEVICE_DT_GET(UART0_DEV_NODE);
const struct device *const uart2_dev = DEVICE_DT_GET(UART2_DEV_NODE);

 /* end UART test */

////#define BUFFER_SIZE 1*5
//#define BUFFER_SIZE 2  //!< number of ADC channels to read
//
//struct gpio_pin 
//{
//	const char * const port;
//	const uint8_t number;
//};
//
//#define ADC_DEVICE_NAME DT_ADC_0_NAME
//#define ADC_RESOLUTION 10
//#define ADC_GAIN ADC_GAIN_1_6
//#define ADC_REFERENCE ADC_REF_INTERNAL
//#define ADC_ACQUISITION_TIME ADC_ACQ_TIME(ADC_ACQ_TIME_MICROSECONDS, 10)
//#define ADC_1ST_CHANNEL_ID 0
//#define ADC_1ST_CHANNEL_INPUT NRF_SAADC_INPUT_AIN0
//#define ADC_2ND_CHANNEL_ID 1
//#define ADC_2ND_CHANNEL_INPUT NRF_SAADC_INPUT_AIN1
//#define ADC_3RD_CHANNEL_ID 2
//#define ADC_3RD_CHANNEL_INPUT NRF_SAADC_INPUT_AIN2
//#define ADC_4TH_CHANNEL_ID 3
//#define ADC_4TH_CHANNEL_INPUT NRF_SAADC_INPUT_AIN3
//#define ADC_5TH_CHANNEL_ID 4
//#define ADC_5TH_CHANNEL_INPUT NRF_SAADC_INPUT_AIN4
//#define ADC_6ST_CHANNEL_ID 5
//#define ADC_6ST_CHANNEL_INPUT NRF_SAADC_INPUT_AIN5
//
//#if 1
//static const struct adc_channel_cfg m_4th_channel_cfg =   //!< ADC config for ADC input
//{
//	.gain = ADC_GAIN,
//	.reference = ADC_REFERENCE,
//	.acquisition_time = ADC_ACQUISITION_TIME,
//	.channel_id = ADC_4TH_CHANNEL_ID,
//#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
//	.input_positive = ADC_4TH_CHANNEL_INPUT,
//#endif
//};
//#endif
//
//#if 0
//static const struct adc_channel_cfg m_3rd_channel_cfg = 
//{
//	.gain = ADC_GAIN,
//	.reference = ADC_REFERENCE,
//	.acquisition_time = ADC_ACQUISITION_TIME,
//	.channel_id = ADC_3RD_CHANNEL_ID,
//#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
//	.input_positive = ADC_3RD_CHANNEL_INPUT,
//#endif
//};
//#endif
//
//#if 1
//static const struct adc_channel_cfg m_2nd_channel_cfg =   //!< ADC config for Battery input
//{
//	.gain = ADC_GAIN,
//	.reference = ADC_REFERENCE,
//	.acquisition_time = ADC_ACQUISITION_TIME,
//	.channel_id = ADC_2ND_CHANNEL_ID,
//#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
//	.input_positive = ADC_2ND_CHANNEL_INPUT,
//#endif
//};
//#endif
//
//#if 0
//static const struct adc_channel_cfg m_1st_channel_cfg = 
//{
//	.gain = ADC_GAIN,
//	.reference = ADC_REFERENCE,
//	.acquisition_time = ADC_ACQUISITION_TIME,
//	.channel_id = ADC_1ST_CHANNEL_ID,
//#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
//	.input_positive = ADC_1ST_CHANNEL_INPUT,
//#endif
//};
//#endif
//
//static int16_t m_sample_buffer[BUFFER_SIZE];  //!< buffer to receive ADC data from configured channels
//
///*!
// * \fn int adc_sample(void)
// *
// * \brief get a single reading from ADC channel
// *
// * will read ADC_4TH_CHANNEL_ID\n
// * resoultion set to ADC_RESOLUTION in millivolts\n
// * will printout floating point value based on 3.6V max\n
// *
// * \return ADC_RESOLUTION value on ADC_4TH_CHANNEL_ID if pass or -1 if failed\n
// */
//int adc_sample(void)
//{
//	int ret;
//
//	const struct adc_sequence sequence = 
//    {
////		.channels = BIT(ADC_1ST_CHANNEL_ID) | BIT(ADC_2ND_CHANNEL_ID) | BIT(ADC_3RD_CHANNEL_ID) | BIT(ADC_4TH_CHANNEL_ID),
//		.channels =  BIT(ADC_2ND_CHANNEL_ID) | BIT(ADC_4TH_CHANNEL_ID),
//		.buffer = m_sample_buffer,
//		.buffer_size = sizeof(m_sample_buffer),
//		.resolution = ADC_RESOLUTION,
//	};
//
//	if (!adc_dev) 
//    {
//		return -1;
//	}
//
//    jfet_dk_set_led(BATMONENB,1);
//    k_sleep(K_MSEC(500));
//// k_sleep(K_MSEC(1000));
//    ret = adc_read(adc_dev, &sequence);
//    if(ret !=0)
//      printk("ADC read err: %d\n", ret);
//
//         /* Print the AIN0 values */
//	for (int i = 0; i < BUFFER_SIZE; i++) 
//    {
////		adc_voltage = (float)(((float)m_sample_buffer[i] / 1023.0f) * 3600.0f);
////		printf("Measured voltage: %3.2f V\n", adc_voltage);
////        printf("den: %3.2f\r\n",((pow(2,ADC_RESOLUTION) -1 ) * 1.0)  * 3600.0f);
//		adc_voltage[i] = (float)(((float)m_sample_buffer[i] / ((pow(2,ADC_RESOLUTION) -1 ) * 1.0) ) * 3600.0f);
////		printf("Measured voltage: %3.2f V\n", adc_voltage);
////        adc_voltage[i] = adc_voltage[i]/1000.0;
//        adc_voltage[i] = adc_voltage[i]/10000.0;
////		printk("ADC raw value: %d 0x%03X\n", m_sample_buffer[i], m_sample_buffer[i]);
////		printf("Measured voltage: %3.2f V\n", adc_voltage[i] * 10);
//	}
//    jfet_dk_set_led(BATMONENB,0);
//
//	return ret;
//}
//
//static const struct device *led_devs[ARRAY_SIZE(led_pins)];  //!< array of device structures from binding to led_pins array

#if 1
struct k_timer verify_json_timer;   //!< timer to verify a PUBACK was seen after a publish operation

 /*!
 * \fn void verify_json_handler(struct k_work *work)
 *
 * \brief work function JSON packet verify handler
 *
 * handles all packets seen on ../update/accepted topic
 * or running mode
 *
 * \param work I2C address
 *
 * \return none
 */
void verify_json_handler(struct k_work *work)
{
    struct tm  time;
    unsigned char buf[80];
    time_t curTime;
    struct tm *pTime = &time;
    uint8_t ret = false;
    char str[140];


    curTime = k_uptime_get();
    date_time_uptime_to_unix_time_ms(&curTime);
    curTime = (curTime/1000) + (flashParameters.timeZone * 60 * 60);       /* convert to seconds */

//    getRTCTime(&time);
//    curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
    pTime = localtime(&curTime);
    sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d",pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
    sprintf(str,"%s verify JSON not seen jsonDataSent: %d\r\n",buf,jsonDataSent);
    myPrintfE(str);
#if 1
//    sendJSONAcknowledged = false;    /* JSON packets not acknowledged */
    if(jsonDataSent)  /* check if this was data packet not seen */
    {
        ret = addSavedJSONPacket(jsonString);   /* add save to packet if not already there and check if should be saved */
        if(ret == true)                         /* save in flash if not already saved */
        {
            sprintf(str,"Adding Saved JSON Packet %d\r\n",flashParameters.savedCount);
            myPrintfW(str);
            saveJSONPacket(jsonString);
        }
        if(flashParameters.savedCount < MAXIMUM_MISSED_IN_A_ROW)
        {
            return;
        }
        else
        {
            if(flashParameters.savedCount%MAXIMUM_MISSED_IN_A_ROW == 0)
            {
                sprintf(str,"Saved JSON Packets exceeded %d in a row. Rebooting!",MAXIMUM_MISSED_IN_A_ROW);
                myPrintfW(str);
                printk("\r\n\n\n");
                k_sleep(K_SECONDS(2));
#if defined(CONFIG_REBOOT)
                sys_reboot(SYS_REBOOT_COLD);
#endif
            }
        }
    }
#endif
}

K_WORK_DEFINE(verify_json_work, verify_json_handler);

 /*!
 * \fn void verify_json_timer_handler(struct k_timer *dummy)
 *
 * \brief JSON saved handler
 *
 * timer handler jfor saved JSOn packets
 *
 * \param dummy pointer to a timer sturcture
 *
 * \return none
 */
void verify_json_timer_handler(struct k_timer *dummy)
{
    k_work_submit(&verify_json_work);
}
#endif

#if 1
struct k_timer json_saved_timer;    //!< timer to check if any JSON packets to be sent

 /*!
 * \fn void json_saved_handler(struct k_work *work)
 *
 * \brief work function JSON packet handler
 *
 * handles all CLi commands when in stopped (idle)
 * or running mode
 *
 * \param work I2C address
 *
 * \return none
 */
void json_saved_handler(struct k_work *work)
{
    char str[80];
    
    /* check if any saved JSON packets or not running */
    if( (flashParameters.savedCount == 0) || (runFlag == 0) || (networkConnected == 0))
        return;
    sprintf(str,"json_saved_handler saved %d sendJSONAcknowledged: %s\r\n",flashParameters.savedCount,sendJSONAcknowledged?"True":"False");
    myPrintfS(str);

#if 1
    if(sendJSONAcknowledged)   /* is it acknokledging */
    {
        if(networkConnectedTest)
        {
            sendSavedJSON = true;
            k_timer_start(&data_sample_timer, K_SECONDS(0), K_SECONDS(flashParameters.dutyCycle));
        }
    }
#endif
}

K_WORK_DEFINE(json_saved_work, json_saved_handler);

 /*!
 * \fn void json_saved_timer_handler(struct k_timer *dummy)
 *
 * \brief JSON saved handler
 *
 * timer handler jfor saved JSOn packets
 *
 * \param dummy pointer to a timer sturcture
 *
 * \return none
 */
void json_saved_timer_handler(struct k_timer *dummy)
{
    k_work_submit(&json_saved_work);
}
#endif

#if 1
struct k_timer cli_command_timer; //!< timer to check for UART 0 (keyboard) input for CLI commands

 /*!
 * \fn void cli_command_handler(struct k_work *work)
 *
 * \brief work function CLI command handler
 *
 * handles all CLi commands when in stopped (idle)
 * or running mode
 *
 * \param work I2C address
 *
 * \return none
 */
void cli_command_handler(struct k_work *work)
{
#if 1
    static uint32_t timerCount;
    struct uart_data_t *buf;

	timerCount++;

	if(!networkConnected)
	{
		if(timerCount & 0x02)
		{
//            sendCommand("vis Home.networkStatus,1");
            NexButton_Set_background_crop_picc(&networkStatus0,6);
            NexButton_Set_background_crop_picc(&networkStatus1,6);
            NexButton_Set_background_crop_picc(&networkStatus2,6);
		}
		else
		{
//            sendCommand("vis networkStatus,0");
            NexButton_Set_background_crop_picc(&networkStatus0,2);
            NexButton_Set_background_crop_picc(&networkStatus1,2);
            NexButton_Set_background_crop_picc(&networkStatus2,2);
		}
		recvRetCommandFinished();
	}

#if 1
    if( (timerCount & 0x7F) == 0)  /* check for dowbnlink or FOTA downlink command */
    {
        if(runFlag)        /* check if network connected */
        {
            time_NTP_server_get();
//          printk("Downlink NTP date %s(UTC)\r\n",sntpTime);
        }
	}
#endif

    if( (timerCount & 0x3FF) == 0)  /* check for day change */
    {
//		LOG_INF("cli_command_handler called %s");
////        checkDayChange();
        if(runFlag)        /* check if network connected */
        {
            checkHourChange();
        }
    }

 	buf = k_fifo_get(&fifo_uart_rx_data, K_NO_WAIT);		/* check if command line seen on UART */
	if (buf) 
	{
		strncpy(cmdLine,buf->data,buf->len);
		cmdLine[buf->len] = '\n';
		cmdLine[buf->len + 1] = '\0';
        k_timer_stop(&cli_wait_timer);          /* stop the wait timer so you stay in CLI */
		executeCmd(cmdLine);
	}

	if(display_line_seen)
	{
//		printk("display_line_seen %d\r\n",num_chars_seen);
		if( (uart2_buf[0] == 0x65) || (uart2_buf[0] == 0x67) || (uart2_buf[0] == 0x68))
		{
//			printk("display_line_seen %d\r\n",num_chars_seen);
//			for(int i=0;i<num_chars_seen; i++)
//			{
//	            printk("%d %02X\r\n",i,uart2_buf[i]);
//			}
			if(num_chars_seen > 7)
				num_chars_seen = 7;
			displayTouchHandler(uart2_buf,num_chars_seen);
			display_line_seen = false;
			num_chars_seen = 0;
			num_end_seen   = 0;
			touchSeen = 0;
		}
	}

#endif
}

K_WORK_DEFINE(cli_command_work, cli_command_handler);

 /*!
 * \fn void cli_command_timer_handler(struct k_timer *dummy)
 *
 * \brief timer CLI command handler
 *
 * timer handler just submits timer work function to the work queue
 *
 * \param dummy pointer to a timer sturcture
 *
 * \return none
 */
void cli_command_timer_handler(struct k_timer *dummy)
{
    k_work_submit(&cli_command_work);
}
#endif

#if 1
struct k_timer cli_wait_timer; //!< timer to wait N seconds before starting application

 /*!
 * \fn void cli_wait_handler(struct k_work *work)
 *
 * \brief work function CLI wait handler
 *
 * handles all CLi commands when in stopped (idle)
 * or running mode
 *
 * \param work I2C address
 *
 * \return none
 */
void cli_wait_handler(struct k_work *work)
{
#if 1
    runFlag = true;
    myPrintfS("Start Cellular Sensor Running (Timeout)\r\n");
#endif
}

K_WORK_DEFINE(cli_wait_work, cli_wait_handler);

 /*!
 * \fn void cli_wait_timer_handler(struct k_timer *dummy)
 *
 * \brief timer CLI command handler
 *
 * timer handler just submits timer work function to the work queue
 *
 * \param dummy pointer to a timer sturcture
 *
 * \return none
 */
void cli_wait_timer_handler(struct k_timer *dummy)
{
    k_work_submit(&cli_wait_work);
}
#endif

#if 1
struct k_timer power_alarm_timer;    //!< timer to check for power alarms

 /*!
 * \fn void power_alarm_handler(struct k_work *work)
 *
 * \brief work function power alarm handler
 *
 * checks for power alarms at poweralarmcheck interval
 * and turns on alarm and restarts data timer to send
 * a JSON packet immediately
 *
 * \param work I2C address
 *
 * \return none
 */
void power_alarm_handler(struct k_work *work)
{
#if 1
//    struct modem_param_info modem_param1;
//    struct tm  time;
//    unsigned char buf[80];
//    time_t curTime;
//    struct tm *pTime = &time;

     /* count down and check for pressure alarms */
    powerAlarm = 0;
    if(flashParameters.alarmsEnabled && runFlag)
    {
        checkPowerAlarmValid();      /* count down any waiting alarms */
 //       powerAlarmCheck = flashParameters.pressureAlarmCheck;
        powerAlarm = checkPowerAlarm();

         /* if network connected get time from modem */
//        if(networkConnected)
//        {
//            modem_info_params_init(&modem_param1);
//            modem_info_params_get(&modem_param1);
//        }

//        getRTCTime(&time);
//        curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
//        pTime = localtime(&curTime);
//        sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d",pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
//        printk("%s Power Alarm Check 0x%02X\r\n",buf,powerAlarm);
//        printk("Power Alarm Check 0x%02X\r\n",powerAlarm);
        printk("Power Alarm Check %d Valid: %d\r\n",powerAlarm,flashParameters.powerAlarms->powerAlarmValid);
        if(powerAlarm != 0)
        {
            strcpy(message,"AC Power Alarm");
            buzzen = 1;

           cliChangedCFG = true;
			 pwm_set_dt(&pwm_led0, DEFAULT_PERIOD_NSEC, DEFAULT_PULSE_NSEC);
           k_timer_start(&data_sample_timer, K_SECONDS(0), K_SECONDS(flashParameters.dutyCycle));
        }
    }
#endif
}

K_WORK_DEFINE(power_alarm_work, power_alarm_handler);

/*!
 * \fn void power_alarm_timer_handler(struct k_timer *dummy)
 *
 * \brief timer poweralarm handler
 *
 * timer handler just submits timer work function to the work queue
 *
 * \param dummy pointer to a timer sturcture
 *
 * \return none
 */
void power_alarm_timer_handler(struct k_timer *dummy)
{
    k_work_submit(&power_alarm_work);
}
#endif

#if 1
struct k_timer pressure_alarm_timer;    //!< timer to check for pressure alarms

 /*!
 * \fn void pressure_alarm_handler(struct k_work *work)
 *
 * \brief work function pressure alarm handler
 *
 * checks for pressure alarms at pressurealarmcheck interval
 * and turns on alarm and restarts data timer to send
 * a JSON packet immediately
 *
 * \param work I2C address
 *
 * \return none
 */
void pressure_alarm_handler(struct k_work *work)
{
//    struct modem_param_info modem_param1;
//    struct tm  time;
//    unsigned char buf[80];
//    time_t curTime;
//    struct tm *pTime = &time;
//    myPrintfW("pressure_alarm_handler\r\n");
     /* count down and check for pressure alarms */
    getSensorPressure();                /* get current pressure */
    pressureAlarm = 0;
    if(flashParameters.alarmsEnabled && runFlag)
    {

        checkPressureAlarmsValid();      /* count down any waiting alarms */
        pressureAlarmCheck = flashParameters.pressureAlarmCheck;
        pressureAlarm = checkPressureAlarms();

         /* if network connected get time from modem */
//        if(networkConnected)
//        {
//            modem_info_params_init(&modem_param1);
//            modem_info_params_get(&modem_param1);
//        }

//        getRTCTime(&time);
//        curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
//        pTime = localtime(&curTime);
//        sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d",pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
//        printk("%s Pressure Alarm Check 0x%02X\r\n",buf,pressureAlarm);
//        printk("Pressure Alarm Check 0x%02X\r\n",pressureAlarm);
        printk("Pressure Alarm Check %d Low Valid: %d High Valid: %d\r\n",pressureAlarm,flashParameters.pressureAlarms->lowPressureAlarmValid, flashParameters.pressureAlarms->highPressureAlarmValid);
        if(pressureAlarm != 0)
        {
            if( (pressureAlarmLow !=0)&& (pressureAlarmHigh !=0) )
                strcpy(message,"Pressure Alarm Low and High");
            else if(pressureAlarmLow !=0)
                strcpy(message,"Pressure Alarm Low");
            else if(pressureAlarmHigh !=0)
                strcpy(message,"Pressure Alarm High");
            buzzen = 1;

			  pwm_set_dt(&pwm_led0, DEFAULT_PERIOD_NSEC, DEFAULT_PULSE_NSEC);
            k_timer_start(&data_sample_timer, K_SECONDS(0), K_SECONDS(flashParameters.dutyCycle));
        }
    }
}

K_WORK_DEFINE(pressure_alarm_work, pressure_alarm_handler);

/*!
 * \fn void pressure_alarm_timer_handler(struct k_timer *dummy)
 *
 * \brief timer pressure alarm handler
 *
 * timer handler just submits timer work function to the work queue
 *
 * \param dummy pointer to a timer sturcture
 *
 * \return none
 */
void pressure_alarm_timer_handler(struct k_timer *dummy)
{
    k_work_submit(&pressure_alarm_work);
}
#endif

#if 1
struct k_timer humidity_alarm_timer;    //!< timer to check for humidity alarms

 /*!
 * \fn void humidity_alarm_handler(struct k_work *work)
 *
 * \brief work function humidity alarm handler
 *
 * checks for humidity alarms at humidityalarmcheck interval
 * and turns on alarm and restarts data timer to send
 * a JSON packet immediately
 *
 * \param work I2C address
 *
 * \return none
 */
void humidity_alarm_handler(struct k_work *work)
{
//    struct modem_param_info modem_param1;
//    struct tm  time;
//    unsigned char buf[80];
//    time_t curTime;
//    struct tm *pTime = &time;

     /* count down and check for humidity alarms */
    getSensorHumidity();                /* get current humidity */
    if(!runFlag)
    {
        return;
    }
    humidityAlarm = 0;
    if(flashParameters.alarmsEnabled && runFlag)
    {
        checkHumidityAlarmsValid();      /* count down any waiting alarms */
        humidityAlarmCheck = flashParameters.humidityAlarmCheck;
        humidityAlarm = checkHumidityAlarms();

         /* if network connected get time from modem */
//        if(networkConnected)
//        {
//            modem_info_params_init(&modem_param1);
//            modem_info_params_get(&modem_param1);
//        }

//        getRTCTime(&time);
//        curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
//        pTime = localtime(&curTime);
//        sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d",pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
//        printk("%s Humidity Alarm Check 0x%02X\r\n",buf,humidityAlarm);
//        printk("Humidity Alarm Check 0x%02X\r\n",humidityAlarm);
        printk("Humidity Alarm Check %d Low Valid: %d High Valid: %d\r\n",humidityAlarm,flashParameters.humidityAlarms->lowHumidityAlarmValid, flashParameters.humidityAlarms->highHumidityAlarmValid);
        if(humidityAlarm != 0)
        {
            if( (humidityAlarmLow !=0)&& (humidityAlarmHigh !=0) )
                strcpy(message,"Humidity Alarm Low and High");
            else if(humidityAlarmLow !=0)
                strcpy(message,"Humidity Alarm Low");
            else if(humidityAlarmHigh !=0)
                strcpy(message,"Humidity Alarm High");
			else
                strcpy(message,"Humidity Alarm Unknown");
            buzzen = 1;
			pwm_set_dt(&pwm_led0, DEFAULT_PERIOD_NSEC, DEFAULT_PULSE_NSEC);
            k_timer_start(&data_sample_timer, K_SECONDS(0), K_SECONDS(flashParameters.dutyCycle));
        }
    }
}

K_WORK_DEFINE(humidity_alarm_work, humidity_alarm_handler);

/*!
 * \fn void humidity_alarm_timer_handler(struct k_timer *dummy)
 *
 * \brief timer humidity alarm handler
 *
 * timer handler just submits timer work function to the work queue
 *
 * \param dummy pointer to a timer sturcture
 *
 * \return none
 */
void humidity_alarm_timer_handler(struct k_timer *dummy)
{
    k_work_submit(&humidity_alarm_work);
}
#endif

#if 1
struct k_timer temp_alarm_timer;    //!< timer to check for temperature alatms

 /*!
 * \fn void temp_alarm_handler(struct k_work *work)
 *
 * \brief work function temperature alarm handler
 *
 * checks for temperature alarms at tempalarmcheck interval
 * and turns on alarm and restarts data timer to send
 * a JSON packet immediately
 *
 * \param work I2C address
 *
 * \return none
 */
void temp_alarm_handler(struct k_work *work)
{
//    static uint8_t timerCount;
//    int err;
//    struct modem_param_info modem_param1;
    unsigned char buf[80];
//    struct tm  time;
//    time_t curTime;
    time_t upTime;
//    struct tm *pTime = &time;
    uint32_t currentPage;
    struct tm *pTime;

    getSensorTemp();                /* get current temp */

    if(!runFlag)
    {
        return;
    }
//    if(networkConnected)
//    {
    modem_info_params_init(&modem_param1);
    modem_info_params_get(&modem_param1);
   	strcpy(modemTime,modem_param1.network.date_time.value_string);
//    }
//    getRTCTime(&time);
//    curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
//    curTime = mktime(&time) ; /*convert to local time */
//    pTime = localtime(&curTime);

        /* get current uptime */
    upTime = k_uptime_get();
    date_time_uptime_to_unix_time_ms(&upTime);
    upTime = (upTime/1000) + (flashParameters.timeZone * 60 * 60);       /* convert to seconds */
    pTime = localtime(&upTime);
    sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d (%d)\r\n",pTime->tm_mon+1,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec, flashParameters.timeZone);
//    sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d (%d)\r\n", time.tm_mon,time.tm_mday,time.tm_year + 2000,time.tm_hour,time.tm_min,time.tm_sec,flashParameters.timeZone);
//    sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d (%d)\r\n", pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 2000,pTime->tm_hour,pTime->tm_min,pTime->tm_sec,flashParameters.timeZone);
//    printk("temp_alarm_handler %s",buf);
     /* get current page and set up page select icons */
    sendCommand("get dp");
    recvRetNumber(&currentPage);
//    printk("current page T: %d\r\n",currentPage);

    switch(currentPage)
    {
        case 0:
              NexText_setText(&text0DateTime,buf);
              break;

        case 1:
              NexText_setText(&text1DateTime,buf);
              break;

        case 2:
              NexText_setText(&text2DateTime,buf);
              break;
    }
    tempAlarm = 0;
    if(flashParameters.alarmsEnabled && runFlag)
    {
        checkTempAlarmsValid();      /* count down any waiting alarms */
        tempAlarm = checkTempAlarms();
//        getRTCTime(&time);
//        curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
//        pTime = localtime(&curTime);
//        sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d",pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
//        printk("%s Temp Alarm Check 0x%02X\r\n",buf,tempAlarm);
//        printk("Temp Alarm Check 0x%02X\r\n",tempAlarm);
        printk("Temp Alarm Check %d Low Valid: %d High Valid: %d\r\n",tempAlarm,flashParameters.tempAlarms->lowTempAlarmValid, flashParameters.tempAlarms->highTempAlarmValid);
        if(tempAlarm != 0)
        {
            if( (tempAlarmLow !=0)&& (tempAlarmHigh !=0) )
                strcpy(message,"Temperature Alarm Low and High");
            else if(tempAlarmLow !=0)
                strcpy(message,"Temperature Alarm Low");
            else if(tempAlarmHigh !=0)
                strcpy(message,"Temperature Alarm High");
            buzzen = 1;
			pwm_set_dt(&pwm_led0, DEFAULT_PERIOD_NSEC, DEFAULT_PULSE_NSEC);
            k_timer_start(&data_sample_timer, K_SECONDS(0), K_SECONDS(flashParameters.dutyCycle));
        }
    }
}

K_WORK_DEFINE(temp_alarm_work, temp_alarm_handler);

/*!
 * \fn void temp_alarm_timer_handler(struct k_timer *dummy)
 *
 * \brief timer temperature alarm handler
 *
 * timer handler just submits timer work function to the work queue
 *
 * \param dummy pointer to a timer sturcture
 *
 * \return none
 */
void temp_alarm_timer_handler(struct k_timer *dummy)
{
    k_work_submit(&temp_alarm_work);
}
#endif

#if 1
struct k_timer ezoec_alarm_timer;    //!< timer to check for EZOEC alarms

 /*!
 * \fn void ezoec_alarm_handler(struct k_work *work)
 *
 * \brief work function EZOEC alarm handler
 *
 * checks for temperature alarms at tempalarmcheck interval
 * and turns on alarm and restarts data timer to send
 * a JSON packet immediately
 *
 * \param work I2C address
 *
 * \return none
 */
void ezoec_alarm_handler(struct k_work *work)
{
//    static uint8_t timerCount;
//    int err;
//    struct modem_param_info modem_param1;
//    unsigned char buf[80];
//    struct tm  time;
//    time_t curTime;
    time_t upTime;
//    struct tm *pTime = &time;
//    uint32_t currentPage;
    struct tm *pTime;

    getSensorEZO();
    if(!runFlag)
    {
        return;
    }
//    if(networkConnected)
//    {
//    modem_info_params_init(&modem_param1);
//    modem_info_params_get(&modem_param1);
//   	strcpy(modemTime,modem_param1.network.date_time.value_string);
//    }
//    getRTCTime(&time);
//    curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
//    curTime = mktime(&time) ; /*convert to local time */
//    pTime = localtime(&curTime);

        /* get current uptime */
    upTime = k_uptime_get();
    date_time_uptime_to_unix_time_ms(&upTime);
    upTime = (upTime/1000) + (flashParameters.timeZone * 60 * 60);       /* convert to seconds */
    pTime = localtime(&upTime);
//    sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d (%d)\r\n",pTime->tm_mon+1,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec, flashParameters.timeZone);
//    sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d (%d)\r\n", time.tm_mon,time.tm_mday,time.tm_year + 2000,time.tm_hour,time.tm_min,time.tm_sec,flashParameters.timeZone);
//    sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d (%d)\r\n", pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 2000,pTime->tm_hour,pTime->tm_min,pTime->tm_sec,flashParameters.timeZone);
//    printk("ezoec_alarm_handler %s",buf);
     /* get current page and set up page select icons */
    ezoAlarm = 0;
    if(flashParameters.alarmsEnabled && runFlag)
    {
        checkEZOAlarmsValid();      /* count down any waiting alarms */
        ezoAlarm = checkEZOAlarms();
//        printk("%s Temp Alarm Check 0x%02X\r\n",buf,ezoAlarm);
        printk("EZO Alarm Check %d Low Valid: %d High Valid: %d\r\n",ezoAlarm,flashParameters.ezoecAlarms->lowezoecAlarmValid, flashParameters.ezoecAlarms->highezoecAlarmValid);
        if(ezoAlarm != 0)
        {
            if( (ezoAlarmLow !=0)&& (ezoAlarmHigh !=0) )
                strcpy(message,"EZO Alarm Low and High");
            else if(ezoAlarmLow !=0)
                strcpy(message,"EZO Alarm Low");
            else if(ezoAlarmHigh !=0)
                strcpy(message,"EZO Alarm High");
            buzzen = 1;
			pwm_set_dt(&pwm_led0, DEFAULT_PERIOD_NSEC, DEFAULT_PULSE_NSEC);
            k_timer_start(&data_sample_timer, K_SECONDS(0), K_SECONDS(flashParameters.dutyCycle));
        }
    }
}

K_WORK_DEFINE(ezoec_alarm_work, ezoec_alarm_handler);

/*!
 * \fn void ezoec_alarm_timer_handler(struct k_timer *dummy)
 *
 * \brief timer EZOEC alarm handler
 *
 * timer handler just submits timer work function to the work queue
 *
 * \param dummy pointer to a timer sturcture
 *
 * \return none
 */
void ezoec_alarm_timer_handler(struct k_timer *dummy)
{
    k_work_submit(&ezoec_alarm_work);
}
#endif

/*!
 * \fn void jfet_init(void)
 *
 * \brief JFET initialization and setup
 *
 *
 * \return None
 */
void jfet_init(void)
{
    uint32_t ret;
    int err;
    int i;
    struct tm  time;
    char buf[100];
//    char buf1[30];
//    uint8_t *pFlashParameters = (uint8_t *)&flashParameters;
//    uint8_t dataIn[256 + 8];
//    time_t curTime;
//    struct tm *pTime = &time;
//    uint8_t flashIDIn[5];
    char str[200];
//    time_t t;
	uint32_t max_period;

    /* Intializes random number generator */
//    srand((unsigned) time(&t));

#if 1
	myPrintfS("Starting Data Logger Rev E\r\n");
	sprintf(str,"Board: %s%s\r\nFW VERSION: V %d.%02d %s%s\r\n",BOARD_NAME, BOARD_NAME_EXT, VERSION_MAJOR, VERSION_MINOR, VERSION_DATE,VERSION_DATE_EXT);
	myPrintfS(str);
	sprintf(str,"Built:%s %s\r\n", __DATE__, __TIME__);
	myPrintfS(str);
    /* find the reason that we started the program */
    ret = NRF_POWER->RESETREAS;
    memset(resetCause,0,256);
    if(ret == 0)
      sprintf(resetCause,"Power applied reset, ");
    if(ret & 0x1)
      strcat(resetCause,"Pin reset, ");
    if(ret & 0x2)
      strcat(resetCause,"Global Watchdog reset, ");
    if(ret & 0x4)
      strcat(resetCause,"Wakeup from system off detect from GPIO reset, ");
    if(ret & 0x10)
      strcat(resetCause,"Wakeup from system off debug interface mode reset, ");
    if(ret & 0x10000)
      strcat(resetCause,"AIRCR.SYSRESETREQ reset, ");
    if(ret & 0x20000)
      strcat(resetCause,"CPU lockup reset, ");
    if(ret & 0x40000)
      strcat(resetCause,"CTRL_AP reset, ");
    if(strlen(resetCause) > 90)
      resetCause[90] = '\0';
//    resetCause[strlen(resetCause) - 2] = '\0';
    NRF_POWER->RESETREAS=ret;
//    snprintf(message,MESSAGESIZE -1,"%s",resetCause);
    strcpy(message,resetCause);
    if (IS_ENABLED(CONFIG_LTE_NETWORK_MODE_LTE_M))
        sprintf(buf,"V %d.%02d %s%s (No GPS)",VERSION_MAJOR, VERSION_MINOR, VERSION_DATE,VERSION_DATE_EXT);
    if (IS_ENABLED(CONFIG_LTE_NETWORK_MODE_LTE_M_GPS))
        sprintf(buf,"V %d.%02d %s%s (GPS)",VERSION_MAJOR, VERSION_MINOR, VERSION_DATE,VERSION_DATE_EXT);
    if (IS_ENABLED(CONFIG_LTE_NETWORK_MODE_NBIOT))
        sprintf(buf,"V %d.%02d %s%s (NB-IOT)",VERSION_MAJOR, VERSION_MINOR, VERSION_DATE,VERSION_DATE_EXT);
    strcat(message," ");
    strcat(message,buf);   /* add in version to initial reset cause */
//    sprintf(str,"Message: %s\r\n",message);
    myPrintfS(str);
    sprintf(str,"*** Reason for reset 0x%0X\r\n*** %s\r\n",ret,resetCause);
    myPrintfS(str);
    sprintf(str,"*** FW VERSION: V %d.%02d %s%s\r\n", VERSION_MAJOR, VERSION_MINOR, VERSION_DATE,VERSION_DATE_EXT);
    myPrintfS(str);
    if (IS_ENABLED(CONFIG_LTE_NETWORK_MODE_LTE_M))
        myPrintfW("*** GPS not available!\r\n");
    if (IS_ENABLED(CONFIG_LTE_NETWORK_MODE_LTE_M_GPS))
        myPrintfS("*** GPS available!\r\n");
    if (IS_ENABLED(CONFIG_LTE_NETWORK_MODE_NBIOT))
        myPrintfS("*** NB-IOT\r\n");
#endif

/* start GPIO test */
	myPrintfS("*** GPIO Init ***\r\n");

	/* set up SW2CTRL as output/pullup */
	if (!device_is_ready(sw2ctrl_dev)) 
	{
		myPrintfW("SW2CTRL device is not ready\r\n");
	}
	else
		myPrintfS("SW2CTRL device is ready\r\n");
	gpio_pin_configure(sw2ctrl_dev, SW2CTRL_DEV_PIN, GPIO_OUTPUT | GPIO_PULL_UP);
	gpio_pin_set(sw2ctrl_dev, SW2CTRL_DEV_PIN, 1);

	/* set up SPI CS2N as output/pullup */
	if (!device_is_ready(spi2_csn_dev)) 
	{
		myPrintfW("SPI CS2N device is not ready\r\n");
	}
	else
		myPrintfS("SPI CS2N device is ready\r\n");
	gpio_pin_configure(spi2_csn_dev, CS2N_DEV_PIN, GPIO_OUTPUT | GPIO_PULL_UP);
	gpio_pin_set(spi2_csn_dev, CS2N_DEV_PIN, 1);

	/* set up SPI CS3N as output/pullup */
	if (!device_is_ready(spi3_csn_dev)) 
	{
		myPrintfW("SPI CS3N device is not ready\r\n");
	}
	else
		myPrintfS("SPI CS3N device is ready\r\n");
	gpio_pin_configure(spi3_csn_dev, CS3N_DEV_PIN, GPIO_OUTPUT | GPIO_PULL_UP);
	gpio_pin_set(spi3_csn_dev, CS3N_DEV_PIN, 1);

	/* set up BAT_MON_EN as output/pullup */
	if (!device_is_ready(bat_mon_en_dev)) 
	{
		myPrintfW("BAT MON EN device is not ready\r\n");
	}
	else
		myPrintfS("BAT MON EN device is ready\r\n");
	gpio_pin_configure(bat_mon_en_dev, BAT_MON_EN_DEV_PIN, GPIO_OUTPUT | GPIO_PULL_UP);
	gpio_pin_set(bat_mon_en_dev, BAT_MON_EN_DEV_PIN, 0);
    batMonEnab = 0;

	/* set up GPIO_09 as input/pullup */
	if (!device_is_ready(gpio_09_dev)) 
	{
		myPrintfW("GPIO_09 device is not ready\r\n");
	}
	else
		myPrintfS("GPIO_09 device is ready\r\n");
	gpio_pin_configure(gpio_09_dev, GPIO_09_DEV_PIN, GPIO_INPUT | GPIO_PULL_UP | GPIO_OUTPUT_HIGH);

	/* set up Charging as input/pullup */
	if (!device_is_ready(chrg_dev)) 
	{
		myPrintfW("CHRG device is not ready\r\n");
	}
	else
		myPrintfS("CHRG device is ready\r\n");
	gpio_pin_configure(chrg_dev, CHRG_DEV_PIN, GPIO_INPUT | GPIO_PULL_UP | GPIO_OUTPUT_HIGH);

	/* set up ACPR as input/pullup */
	if (!device_is_ready(acpr_dev)) 
	{
		myPrintfW("ACPR device is not ready\r\n");
	}
	else
		myPrintfS("ACPR device is ready\r\n");
	gpio_pin_configure(acpr_dev, ACPR_DEV_PIN, GPIO_INPUT | GPIO_PULL_UP | GPIO_OUTPUT_HIGH);

	/* set up PWM */
	printk("Calibrating PWM for channel %d...\n", pwm_led0.channel);
	max_period = MAX_PERIOD;
	pwm_set_dt(&pwm_led0, 0, 0); 		/* turn off PWM */

#if 0
	int test = 0;
	while(1)
	{
		printf("test GPIO SW2CTRL\r\n");
		/* toggle test */
//		gpio_pin_toggle(sw2ctrl_dev, SW2CTRL_DEV_PIN);
//		k_sleep(K_MSEC(10));

		/* set/clear test */
		if(test++ & 0x01)
		{
//			printf("set\r\n");
			gpio_pin_set(sw2ctrl_dev, SW2CTRL_DEV_PIN, 1);
		}
		else
		{
//			printf("clear\r\n");
			gpio_pin_set(sw2ctrl_dev, SW2CTRL_DEV_PIN, 0);
		}
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	int test = 0;
	while(1)
	{
		printf("test GPIO CS2N\r\n");
		/* toggle test */
//		gpio_pin_toggle(spi2_csn_dev, CS2N_DEV_PIN);
//		k_sleep(K_MSEC(10));

		/* set/clear test */
		if(test++ & 0x01)
		{
//			printf("set\r\n");
			gpio_pin_set(spi2_csn_dev, CS2N_DEV_PIN, 1);
		}
		else
		{
//			printf("clear\r\n");
			gpio_pin_set(spi2_csn_dev, CS2N_DEV_PIN, 0);
		}
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	int test = 0;
	while(1)
	{
		printf("test GPIO CS3N\r\n");
		/* toggle test */
//		gpio_pin_toggle(spi3_csn_dev, CS3N_DEV_PIN);
//		k_sleep(K_MSEC(10));

		/* set/clear test */
		if(test++ & 0x01)
		{
//			printf("set\r\n");
			gpio_pin_set(spi3_csn_dev, CS3N_DEV_PIN, 1);
		}
		else
		{
//			printf("clear\r\n");
			gpio_pin_set(spi3_csn_dev, CS3N_DEV_PIN, 0);
		}
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	int test = 0;
	while(1)
	{
		printf("test GPIO BAT_MON_EN %d\r\n",test & 0x01);
		/* toggle test */
//		gpio_pin_toggle(bat_mon_en_dev, BAT_MON_EN_DEV_PIN);
//		k_sleep(K_MSEC(10));

		/* set/clear test */
		if(test++ & 0x01)
		{
//			printf("set\r\n");
			gpio_pin_set(bat_mon_en_dev, BAT_MON_EN_DEV_PIN, 1);
		}
		else
		{
//			printf("clear\r\n");
			gpio_pin_set(bat_mon_en_dev, BAT_MON_EN_DEV_PIN, 0);
		}
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	uint32_t period;
	uint8_t dir = 0U;
	int retx;

	printk("PWM-based blinky\n");

	if (!pwm_is_ready_dt(&pwm_led0)) {
		printk("Error: PWM device %s is not ready\n",
		       pwm_led0.dev->name);
		return 0;
	}

	/*
	 * In case the default MAX_PERIOD value cannot be set for
	 * some PWM hardware, decrease its value until it can.
	 *
	 * Keep its value at least MIN_PERIOD * 4 to make sure
	 * the sample changes frequency at least once.
	 */
	printk("Calibrating for channel %d...\n", pwm_led0.channel);
	max_period = MAX_PERIOD;
	while (pwm_set_dt(&pwm_led0, max_period, max_period / 2U)) {
		max_period /= 2U;
		if (max_period < (4U * MIN_PERIOD)) {
			printk("Error: PWM device "
			       "does not support a period at least %lu\n",
			       4U * MIN_PERIOD);
			return 0;
		}
	}

	printk("Done calibrating; maximum/minimum periods %u/%lu nsec\n",
	       max_period, MIN_PERIOD);

	period = max_period;
	while (1) {
		retx = pwm_set_dt(&pwm_led0, period, period / 2U);
		if (retx) {
			printk("Error %d: failed to set pulse width\n", retx);
			return 0;
		}

		period = dir ? (period * 2U) : (period / 2U);
		if (period > max_period) {
			period = max_period / 2U;
			dir = 0U;
		} else if (period < MIN_PERIOD) {
			period = MIN_PERIOD * 2U;
			dir = 1U;
		}

		k_sleep(K_SECONDS(4U));
	}
#endif

#if 0
	int test = 0;
	while(1)
	{
		printf("test GPIO PWM %d\r\n",test & 0x01);

		/* set/clear test */
		if(test++ & 0x01)
		{
//			printf("set\r\n");
			pwm_set_dt(&pwm_led0, max_period, max_period / 2U); 
		}
		else
		{
//			printf("clear\r\n");
			pwm_set_dt(&pwm_led0, 0, 0); 
		}
		k_sleep(K_MSEC(3000));
	}
#endif

#if 0
	int test = 0;
	struct gpio_callback gpio_09_cb;
	gpio_init_callback(&gpio_09_cb, (gpio_callback_handler_t)gpio09Callback, BIT(GPIO_09_DEV_PIN));
	gpio_add_callback(gpio_09_dev, &gpio_09_cb);
	gpio_pin_interrupt_configure(gpio_09_dev, GPIO_09_DEV_PIN, GPIO_INT_EDGE_BOTH);

	printf("Start GPIO_09 Interrupt test\r\n");
	while(1)
	{
		test = gpio_pin_get(gpio_09_dev, GPIO_09_DEV_PIN);
		printf("GPIO_09 %d\r\n",test);
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	int test = 0;
	while(1)
	{
		test = gpio_pin_get(gpio_09_dev, GPIO_09_DEV_PIN);
		printf("GPIO_09 %d\r\n",test);
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	int test = 0;
	struct gpio_callback chrg_cb;
	gpio_init_callback(&chrg_cb, (gpio_callback_handler_t)chrgCallback, BIT(CHRG_DEV_PIN));
	gpio_add_callback(chrg_dev, &chrg_cb);
	gpio_pin_interrupt_configure(chrg_dev, CHRG_DEV_PIN, GPIO_INT_EDGE_BOTH);

	printf("Start CHRG Interrupt test\r\n");
	while(1)
	{
		test = gpio_pin_get(chrg_dev, CHRG_DEV_PIN);
		printf("CHRG %d\r\n",test);
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	int test = 0;
	while(1)
	{
		test = gpio_pin_get(chrg_dev, CHRG_DEV_PIN);
		printf("CHRG %d\r\n",test);
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	int test = 0;
	struct gpio_callback acpr_cb;
	gpio_init_callback(&acpr_cb, (gpio_callback_handler_t)acprCallback, BIT(ACPR_DEV_PIN));
	gpio_add_callback(acpr_dev, &acpr_cb);
	gpio_pin_interrupt_configure(acpr_dev, ACPR_DEV_PIN, GPIO_INT_EDGE_BOTH);

	printf("Start ACPR Interrupt test\r\n");
	while(1)
	{
		test = gpio_pin_get(acpr_dev, ACPR_DEV_PIN);
		printf("ACPR %d\r\n",test);
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	int test = 0;
	while(1)
	{
		test = gpio_pin_get(acpr_dev, ACPR_DEV_PIN);
		printf("\r\nACPR %d\r\n",test);
		test = gpio_pin_get(chrg_dev, CHRG_DEV_PIN);
		printf("CHRG %d\r\n",test);
		k_sleep(K_MSEC(1000));
	}
#endif

/* end GPIO test */

/* start ADC test */

	myPrintfS("*** ADC Init ***\r\n");

	if (!device_is_ready(adc_dev)) 
	{
		myPrintfW("ADC device is not ready\r\n");
	}
	else
		myPrintfS("ADC device is ready\r\n");

    err = adc_channel_setup(adc_dev, &m_2nd_channel_cfg);
    if (err) 
    {
		myPrintfW("adc setup: 2nd channel failed\n");
    }
	else			
     	myPrintfS("adc setup: 2nd channel passed\n");

   	err = adc_channel_setup(adc_dev, &m_4th_channel_cfg);
    if (err) 
		myPrintfW("adc setup: 4th channel failed\n");
	else			
    	myPrintfS("adc setup: 4th channel passed\n");

#if 0
	while(1)
	{
		err = adc_sample();
		if (err) 
		{
			printf("Error in adc sampling: %d\n", err);
		}
		printf("BAT: %3.2f V ADC: %3.2f V\r\n",adc_voltage[0] * 10,adc_voltage[1] * 10);
		k_sleep(K_MSEC(1000));
	}
#endif

/* end ADC test */

/* start SPI test */

	myPrintfS("*** SPI Init ***\r\n");

	if (!device_is_ready(spi_dev)) 
	{
		myPrintfW("SPI device is not ready\r\n");
	}
	else
		myPrintfS("SPI device is ready\r\n");

#if 0
	int retSPI;
	uint8_t buffer_tx[BUF_SIZE]= {};   //!< SPI transmit buffer
	uint8_t buffer_rx[BUF_SIZE]= {};   //!< SPI receive buffer0
	
	const struct spi_cs_control spi_cs2 =
	{
		.gpio = GPIO_DT_SPEC_GET_OR(CS2N_DEV_NODE, gpios,{0}),
		.delay = 0,
	};

	struct spi_config spi_cfg2 =       //!< SPI configuration
	{
			.frequency = SLOW_FREQ,
			.operation = SPI_OP_MODE_MASTER | SPI_MODE_CPOL |	SPI_MODE_CPHA | SPI_WORD_SET(8) | SPI_LINES_SINGLE,
			.slave = SPI_SLAVE,
	//      .cs = &spi_cs2,
	//      .cs = &spi_cs,
			.cs = NULL,
	};
	struct spi_buf tx_bufs[] =    //!< SPI TX buffer structure
	{
			{
					.buf = buffer_tx,
					.len = BUF_SIZE,
			},
	};
	struct spi_buf rx_bufs[] =     //!< SPI RX buffer structure
	{
			{
					.buf = buffer_rx,
					.len = BUF_SIZE,
			},
	};
	const struct spi_buf_set tx =      //!< SPI TX buffer structure
	{
			.buffers = tx_bufs,
			.count = ARRAY_SIZE(tx_bufs)
	};
	const struct spi_buf_set rx =      //!< SPI RX buffer structure
	{
			.buffers = rx_bufs,
			.count = ARRAY_SIZE(rx_bufs)
	};

	while(1)
	{

		/* zero buffer and load the command */
		for(int i=0;i<4;i++)
		{
			buffer_tx[i] = 0;
			buffer_rx[i] = 0;
		}
		buffer_tx[0] = 0x9F;
		tx_bufs[0].len = 4;
		rx_bufs[0].len = 4;
 
    /* enable CS and send command  */
		gpio_pin_set(spi2_csn_dev, CS2N_DEV_PIN, 0);
     	retSPI = spi_transceive(spi_dev, &spi_cfg2, &tx, &rx);
//		k_sleep(K_MSEC(2));
		gpio_pin_set(spi2_csn_dev, CS2N_DEV_PIN, 1);
		printf("SPI CS2N Test %d 0x%02X 0x%02X 0x%02X\r\n",retSPI, buffer_rx[1], buffer_rx[2], buffer_rx[3]);
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	float tempSPI = 0.0;
	uint16_t tempSPIInt = 0;
	int retSPI;
	uint8_t buffer_tx[BUF_SIZE]= {};   //!< SPI transmit buffer
	uint8_t buffer_rx[BUF_SIZE]= {};   //!< SPI receive buffer0
	
	const struct spi_cs_control spi_cs3 =
	{
		.gpio = GPIO_DT_SPEC_GET_OR(CS3N_DEV_NODE, gpios,{0}),
		.delay = 0,
	};

	struct spi_config spi_cfg3 =       //!< SPI configuration
	{
			.frequency = SLOW_FREQ,
			.operation = SPI_OP_MODE_MASTER | SPI_MODE_CPOL |	SPI_MODE_CPHA | SPI_WORD_SET(8) | SPI_LINES_SINGLE,
			.slave = SPI_SLAVE,
	//      .cs = &spi_cs3,
	//      .cs = &spi_cs,
			.cs = NULL,
	};
	struct spi_buf tx_bufs[] =    //!< SPI TX buffer structure
	{
			{
					.buf = buffer_tx,
					.len = BUF_SIZE,
			},
	};
	struct spi_buf rx_bufs[] =     //!< SPI RX buffer structure
	{
			{
					.buf = buffer_rx,
					.len = BUF_SIZE,
			},
	};
	const struct spi_buf_set tx =      //!< SPI TX buffer structure
	{
			.buffers = tx_bufs,
			.count = ARRAY_SIZE(tx_bufs)
	};
	const struct spi_buf_set rx =      //!< SPI RX buffer structure
	{
			.buffers = rx_bufs,
			.count = ARRAY_SIZE(rx_bufs)
	};

	for(int j=0;j<2;j++)
	{
		/* zero buffer and load the command */
		for(int i=0;i<4;i++)
		{
			buffer_tx[i] = 0;
			buffer_rx[i] = 0;
		}
		buffer_tx[0] = 0x80;		/* set config register */
		buffer_tx[1] = 0xD0;
		tx_bufs[0].len = 2;
		rx_bufs[0].len = 2;
		gpio_pin_set(spi3_csn_dev, CS3N_DEV_PIN, 0);
     	retSPI = spi_transceive(spi_dev, &spi_cfg3, &tx, &rx);
		gpio_pin_set(spi3_csn_dev, CS3N_DEV_PIN, 1);
	}
	k_sleep(K_MSEC(100));

	while(1)
	{

		/* zero buffer and load the command */
		for(int i=0;i<4;i++)
		{
			buffer_tx[i] = 0;
			buffer_rx[i] = 0;
		}
		buffer_tx[0] = 0x01;
		tx_bufs[0].len = 3;
		rx_bufs[0].len = 3;
 
    /* enable CS and send command  */
		gpio_pin_set(spi3_csn_dev, CS3N_DEV_PIN, 0);
     	retSPI = spi_transceive(spi_dev, &spi_cfg3, &tx, &rx);
//		k_sleep(K_MSEC(2));
		gpio_pin_set(spi3_csn_dev, CS3N_DEV_PIN, 1);
		tempSPIInt = ((buffer_rx[1] << 8) | (buffer_rx[2] << 0)) >> 1;
		tempSPI = (tempSPIInt /32768.0) * 430.0;
		printk("SPI CS3N Test %d 0x%02X 0x%02X 0x%04X %d %3.2f\r\n",retSPI, buffer_rx[1], buffer_rx[2], tempSPIInt, tempSPIInt, tempSPI);
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	float temp;

	rtdWrite(0,0xD0);
	k_sleep(K_MSEC(100));

	while(1)
	{ 
    	temp = getRTDTemp();
    	printf("RTD: %3.2f\r\n",temp);
		k_sleep(K_MSEC(1000));
	}
#endif

/* end SPi test */

#if 1
/* start I2C test */

	myPrintfS("*** I2C Init ***\r\n");

	if (!device_is_ready(i2c_dev)) 
	{
		myPrintfW("I2C device is not ready\r\n");
	}
	else
		myPrintfS("I2C device is ready\r\n");

	/* 1. Verify i2c_configure() */
	if (i2c_configure(i2c_dev, i2c_cfg)) 
	{
		myPrintfW("I2C config failed\n");
	}
	else
		myPrintfS("I2C config passed\n");

#if 0
	float temp;

	while(1)
	{
	    mcp3425Write(0x18);
		temp = getThermistorTemp3425();
		printk("Temp: %3.2f\r\n",temp);

//		datas[0] = 0x18;
//		i2c_write(i2c_dev, datas, 1, 0x68);
//		/* 4. verify i2c_read( */
//		datas[0] = 0x00;
//		i2c_write(i2c_dev, datas, 3, 0x68);
//		i2c_read(i2c_dev, datas, 3, 0x68);
//		printf("I2C 0x%02X 0x%02X 0x%02X\r\n",datas[0],datas[1],datas[2]);
		k_sleep(K_MSEC(1000));
	}
#endif

#if 0
	while(1)
	{
	    EZOHUM_Init();
		k_sleep(K_MSEC(1000));
	}
#endif

/* end I2C test */
#endif

/* start UART test*/

	myPrintfS("*** UART0 Init ***\r\n");

	if (!device_is_ready(uart0_dev)) 
	{
		myPrintfW("UART0 device is not ready\r\n");
	}
	else
		myPrintfS("UART0 device is ready\r\n");

	if (!device_is_ready(uart2_dev)) 
	{
		myPrintfW("UART2 device is not ready\r\n");
	}
	else
		myPrintfS("UART2 device is ready\r\n");

    /* enable Uart_0 interrupts so UART_0 */
	uart_irq_callback_set(uart0_dev, uart0_cb);
	uart_irq_rx_enable(uart0_dev);
    uart_irq_tx_disable(uart0_dev);
	
   /* enable Uart_2 interrupts so UART_2 */
	uart_irq_callback_set(uart2_dev, uart2_cb);
	uart_irq_rx_enable(uart2_dev);
//	uart_irq_rx_disable(uart2_dev);
    uart_irq_tx_disable(uart2_dev);

#if 0
/* get config info */

struct uart_config cfg_uart0;
struct uart_config cfg_uart2;

int ret0x = uart_config_get(uart0_dev, &cfg_uart0);
int ret2x = uart_config_get(uart2_dev, &cfg_uart2);

printk("\tUART0\tUART2\r\n");
printk("Baud\t%d\t%d\r\n",cfg_uart0.baudrate,cfg_uart2.baudrate);
printk("Parity\t%d\t%d\r\n",cfg_uart0.parity,cfg_uart2.parity);
printk("Stop\t%d\t%d\r\n",cfg_uart0.stop_bits,cfg_uart2.stop_bits);
printk("data\t%d\t%d\r\n",cfg_uart0.data_bits,cfg_uart2.data_bits);
printk("flow\t%d\t%d\r\n",cfg_uart0.flow_ctrl,cfg_uart2.flow_ctrl);
#endif

#if 0
/* test of TX to UART2 */
    char testStr[] = "uart2 test\r\n";

	while(1)
	{
		myPrintnextIon(testStr);
		k_sleep(K_MSEC(1000));
 	}
#endif

#if 0
/* test of TX/RX polling to UART2 */
	uint8_t ch;
	int retx;
	
	while(1)
	{
		retx = uart_poll_in(uart2_dev,&ch);
		if(retx == 0)
		{
			uart_poll_out(uart2_dev,ch);
			myPrintnextIon("\r\n");
			printf("0x%02X %c\r\n",ch,ch);
		}
 	}
#endif

#if 0
	/* test of RX interrupt from UART0 */
    struct uart_data_t *bufx;
	uint8_t uart0In[30];

	while(1)
	{
		printk("UART0 RX test\r\n");
		bufx = k_fifo_get(&fifo_uart_rx_data, K_FOREVER);		/* check if command line seen on UART */
		if (bufx) 
		{
			strncpy(uart0In,bufx->data,bufx->len);
			uart0In[bufx->len] = '\n';
			uart0In[bufx->len + 1] = '\0';
			printk("Uart0In: %d %s",strlen(uart0In),uart0In);
		}
 	}
#endif

#if 0
	/* test of RX interrupt from UART2 */
    struct uart_data_t *bufx;
	uint8_t uart2In[30];

	while(1)
	{
		printk("UART2 RX test\r\n");
		bufx = k_fifo_get(&fifo_uart2_rx_data, K_FOREVER);		/* check if command line seen on UART */
		if (bufx) 
		{
			strncpy(uart2In,bufx->data,bufx->len);
			uart2In[bufx->len] = '\n';
			uart2In[bufx->len + 1] = '\0';
			printk("Uart2In: %d %s",strlen(uart2In),uart2In);
		}
 	}
#endif
/* end UART test */

//    /* initialize GPIO */
//    jfet_dk_set_led(MEMCS, 0x1);
//    memCS = 1;
//    jfet_dk_set_led(I2CSDA, 0x1);
//    jfet_dk_set_led(I2CSCK, 0x1);
//    jfet_dk_set_led(RTDCS, 0x1);
//    rtdCS = 1;
    
    /* check for I2C and SPI devices */
    rescan();

    /* dummy read to set up flash SPI signals ignore warning message */
//    readNVFlashxID(flashIDIn);
//    writeNVFlashxStatus();
//    readNVFlashxID(flashIDIn);
#if 0
    for(i=0;i < 5; i++)
    {
      printk("i: %d %02X\r\n",i,flashIDIn[i]);
    }
      printk("\r\n");
#endif

    for(i=0; i < MAX_NUMBER_TEMPERATURE_SENSORS; i++)
    {
        tempSensorsPresent[i] = TEMP_NONE;
    }

    for(i=0; i < MAX_NUMBER_HUMIDITY_SENSORS; i++)
    {
        humiditySensorsPresent[i] = HUMIDITY_NONE;
    }

    for(i=0; i < MAX_NUMBER_PRESSURE_SENSORS; i++)
    {
        pressureSensorsPresent[i] = PRESSURE_NONE;
    }

    for(i=0; i < MAX_NUMBER_EZO_SENSORS; i++)
    {
        ezoecSensorsPresent[i] = EZO_NONE;
    }

    /* initialize the Nextion display */
    nextionNextIn = 0;
    nextionNextOut = 0;
    nextionCount = 0;

    CreateNexObject(probe2HighAlarm0, probe2HighAlarm0PageID, probe2HighAlarm0ID, "Home.probeAlarmHi2");
    CreateNexObject(probe2HighAlarm1, probe2HighAlarm1PageID, probe2HighAlarm1ID, "HTS.probeAlarmHi2");
    CreateNexObject(probe2HighAlarm2, probe2HighAlarm2PageID, probe2HighAlarm2ID, "PDP.probeAlarmHi2");
    CreateNexObject(probe2LowAlarm0, probe2LowAlarm0PageID, probe2LowAlarm0ID, "Home.probeAlarmLow2");
    CreateNexObject(probe2LowAlarm1, probe2LowAlarm1PageID, probe2LowAlarm1ID, "HTS.probeAlarmLow2");
    CreateNexObject(probe2LowAlarm2, probe2LowAlarm2PageID, probe2LowAlarm2ID, "PDP.probeAlarmLow2");
    CreateNexObject(probe1HighAlarm0, probe1HighAlarm0PageID, probe1HighAlarm0ID, "Home.probeAlarmHi1");
    CreateNexObject(probe1HighAlarm1, probe1HighAlarm1PageID, probe1HighAlarm1ID, "HTS.probeAlarmHi1");
    CreateNexObject(probe1HighAlarm2, probe1HighAlarm2PageID, probe1HighAlarm2ID, "PDP.probeAlarmHi1");
    CreateNexObject(probe1LowAlarm0, probe1LowAlarm0PageID, probe1LowAlarm0ID, "Home.probeAlarmLow1");
    CreateNexObject(probe1LowAlarm1, probe1LowAlarm1PageID, probe1LowAlarm1ID, "HTS.probeAlarmLow1");
    CreateNexObject(probe1LowAlarm2, probe1LowAlarm2PageID, probe1LowAlarm2ID, "PDP.probeAlarmLow1");
    CreateNexObject(globalAlarm0, globalAlarm0PageID, globalAlarm0ID, "Home.globalAlarm");
    CreateNexObject(globalAlarm1, globalAlarm1PageID, globalAlarm1ID, "HTS.globalAlarm");
    CreateNexObject(globalAlarm2, globalAlarm2PageID, globalAlarm2ID, "PDP.globalAlarm");
    CreateNexObject(tempUnits0, tempUnits0PageID, tempUnits0ID, "Home.tempunits");
    CreateNexObject(tempUnits1, tempUnits1PageID, tempUnits1ID, "HTS.tempunits");
    CreateNexObject(tempUnits2, tempUnits2PageID, tempUnits2ID, "PDP.tempunits");
    CreateNexObject(buzzer0, buzzer0PageID, buzzer0ID, "Home.buzzer");
    CreateNexObject(buzzer1, buzzer1PageID, buzzer1ID, "HTS.buzzer");
    CreateNexObject(buzzer2, buzzer2PageID, buzzer2ID, "PDP.buzzer");
    CreateNexObject(batteryStatus0, batteryStatus0PageID, batteryStatus0ID, "Home.batteryStatus");
    CreateNexObject(batteryStatus1, batteryStatus1PageID, batteryStatus1ID, "HTS.batteryStatus");
    CreateNexObject(batteryStatus2, batteryStatus2PageID, batteryStatus2ID, "PDP.batteryStatus");
    CreateNexObject(networkStatus0, networkStatus0PageID, networkStatus0ID, "Home.networkStatus");
    CreateNexObject(networkStatus1, networkStatus1PageID, networkStatus1ID, "HTS.networkStatus");
    CreateNexObject(networkStatus2, networkStatus2PageID, networkStatus2ID, "PDP.networkStatus");
    CreateNexObject(acpwrStatus0, acpwrStatus0PageID, acpwrStatus0ID, "Home.acpwrStatus");
    CreateNexObject(acpwrStatus1, acpwrStatus1PageID, acpwrStatus1ID, "HTS.acpwrStatus");
    CreateNexObject(acpwrStatus2, acpwrStatus2PageID, acpwrStatus2ID, "PDP.acpwrStatus");
    CreateNexObject(text0IMEI_ID, text0IMEI_IDPageID, text0IMEI_IDID, "Home.IMEI_ID");
    CreateNexObject(text1IMEI_ID, text1IMEI_IDPageID, text1IMEI_IDID, "HTS.IMEI_ID");
    CreateNexObject(text2IMEI_ID, text2IMEI_IDPageID, text2IMEI_IDID, "PDP.IMEI_ID");
    CreateNexObject(text0DateTime, text0DateTimePageID, text0DateTimeID, "Home.dateTime");
    CreateNexObject(text1DateTime, text1DateTimePageID, text1DateTimeID, "HTS.dateTime");
    CreateNexObject(text2DateTime, text2DateTimePageID, text2DateTimeID, "PDP.dateTime");
    CreateNexObject(versionNumber, versionNumberPageID, versionNumberID, "Home.versionNumber");
    CreateNexObject(versionDate, versionDatePageID, versionDateID, "Home.versionDate");
    CreateNexObject(probe1, probe1PageID, probe1ID, "Home.p1Temp");
    CreateNexObject(probe1Min, probe1MinPageID, probe1MinID, "Home.p1Min");
    CreateNexObject(probe1Max, probe1MaxPageID, probe1MaxID, "Home.p1Max");
    CreateNexObject(probe2, probe2PageID, probe2ID, "Home.p2Temp");
    CreateNexObject(probe2Min, probe2MinPageID, probe2MinID, "Home.p2Min");
    CreateNexObject(probe2Max, probe2MaxPageID, probe2MaxID, "Home.p2Max");
    CreateNexObject(probe3, probe3PageID, probe3ID, "HTS.p3Temp");
    CreateNexObject(probe3Min, probe3MinPageID, probe3MinID, "HTS.p3TempMin");
    CreateNexObject(probe3Max, probe3MaxPageID, probe3MaxID, "HTS.p3TempMax");
    CreateNexObject(humid3, humid3PageID, humid3ID, "HTS.p3Humid");
    CreateNexObject(humid3Min, humid3MinPageID, humid3MinID, "HTS.p3HumidMin");
    CreateNexObject(humid3Max, humid3MaxPageID, humid3MaxID, "HTS.p3HumidMax");
    CreateNexObject(probe4, probe4PageID, probe4ID, "PDP.p4Temp");
    CreateNexObject(probe4Min, probe4MinPageID, probe4MinID, "PDP.p4TempMin");
    CreateNexObject(probe4Max, probe4MaxPageID, probe4MaxID, "PDP.p4TempMax");
    CreateNexObject(pressure4, pressure4PageID, pressure4ID, "PDP.p4Pressure");
    CreateNexObject(pressure4Min, pressure4MinPageID, pressure4MinID, "PDP.p4PressureMin");
    CreateNexObject(pressure4Max, pressure4MaxPageID, pressure4MaxID, "PDP.p4PressureMax");
    myPrintfS("Nextion display - initialize display\r\n");
//#if 1
////    while(1)
////    {
//        sendCommand("bkcmd=0");         /* send back code to all commands */
//        recvRetCommandFinished();
////        k_sleep(K_MSEC(3000));
////    }

//#if 0
//float pressure;
//float temperature;
//int retx;
//
//    while(1)
//    {
//        retx = getMSP300PressureTemp(&pressure, &temperature);
//        printk("ret: %d floatP: %3.2f floatT: %3.2f\r\n",retx,pressure,temperature);
//         k_sleep(K_MSEC(1000));
//     }
//#endif

#if 0
char strx[30];
int cnt;
    while(1)
    {
        cnt = NexText_getText(&versionNumber,strx,9);
        strx[16] = '\0';
        printk("Display Version %s cnt: %d\r\n",&strx[1],cnt);
        for(int i=0;i<cnt;i++)
          printk("%d %02X %c\r\n",i,strx[i],strx[i]);
        k_sleep(K_MSEC(1000));
     }
#endif

    sendCommand("sleep=0");         /* send back a replay to all commands */
    recvRetCommandFinished();
    sendCommand("sendxy=0");        /* send back touch coordinates */
    recvRetCommandFinished();
    sendCommand("page Home");        /* send back touch coordinates */
    recvRetCommandFinished();
//    sendCommand("thsp=60*10");        /* go to sleep in N seconds if no touch */
    sendCommand("thsp=0");        /* do not go to sleep if no touch */
    recvRetCommandFinished();
    sendCommand("thup=1");        /* wake on touch */
    recvRetCommandFinished();
//    printk("Nextion display not used\r\n");

    sprintf(str,"Read Flash Parameters size: %d\r\n",sizeof(flashParametersStruct));
	myPrintfS(str);
	getFlashparameters();
	printk("key: 0x%08X\r\n",flashParameters.key);
#if 1
   /* check if any flash parameters have been written */
   if(flashParameters.key == FLASHPARAMETERKEY)
   {
		k_sleep(K_SECONDS(1));
		displayFlashxParameters();
		gpio_pin_set(sw2ctrl_dev, SW2CTRL_DEV_PIN, flashParameters.sw2);
//       if(flashParameters.gpsExternal == 0)
//            jfet_dk_set_led(GPSENB,flashParameters.gpsEnable?1:0);
//       else
//            jfet_dk_set_led(GPSENB,0);
//       jfet_dk_set_led(MODENPMENB,flashParameters.modenPMEnable?1:0);

       /* set EDRX parameters from flash */
//       strcpy(edrx_param,flashParameters.edrx_param);
//YYYY       strcpy(ptw_param,flashParameters.ptw_param);

        /* set PSM parameters from flash */
        strcpy(psm_param_rptau,flashParameters.psm_param_rptau);
        strcpy(psm_param_rat,flashParameters.psm_param_rat);

       tempAlarmValid = flashParameters.tempAlarmValid;
       humidityAlarmValid = flashParameters.humidityAlarmValid;
       pressureAlarmValid = flashParameters.pressureAlarmValid;
       powerAlarmValid = flashParameters.powerAlarmValid;
       ezoecAlarmValid = flashParameters.ezoecAlarmValid;
	   if(flashParameters.dateSet)
	   {
          getRTCTime(&time);
          printk("RTC   date-time: %02d/%02d/%04d %02d:%02d:%02d (%d UTC)\r\n",time.tm_mon,time.tm_mday,time.tm_year + 2000,time.tm_hour,time.tm_min,time.tm_sec,flashParameters.timeZone);
          time_t curTime;
          struct tm *pTime = &time;
//          curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
          curTime = mktime(&time) ; /*convert to local time */
          pTime = localtime(&curTime);
          printk("Local date-time: %02d/%02d/%04d %02d:%02d:%02d\r\n", pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 2000,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
          lastDay = time.tm_mday;
          lastHour = time.tm_hour;
          sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d (%d)\r\n", pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 2000,pTime->tm_hour,pTime->tm_min,pTime->tm_sec,flashParameters.timeZone);
          NexText_setText(&text0DateTime,buf);
          NexText_setText(&text1DateTime,buf);
          NexText_setText(&text2DateTime,buf);
       }
       else
       {
          printk("RTC date-time not set!\r\n");
          NexText_setText(&text0DateTime,"RTC date-time not set!\r\n");
       }

//#if nextionDisplayAvailable
//          myPrintfS("Nextion display - initialize global alarm from flash\r\n");
//#endif
   }
   else
   {
       sprintf(str,"Flash Parameter Key: %08X Not Set\r\n",flashParameters.key);
       myPrintfW(str);
       flashxClearParameters();
       tempAlarmValid = DEFAULT_TEMPALARMVALID;
       humidityAlarmValid = DEFAULT_HUMIDITYALARMVALID;
       pressureAlarmValid = DEFAULT_PRESSUREALARMVALID;
       powerAlarmValid = DEFAULT_POWERALARMVALID;
       ezoecAlarmValid = DEFAULT_EZOECALARMVALID;
      setFlashxParameters(true);
   }
#endif

    /* check for I2C and SPI devices */
    rescan();
//#if nextionDisplayAvailable
//    myPrintfS("Nextion display - initialize buzzer\r\n");
//    myPrintfS("Nextion display - initialize battery status\r\n");
//    myPrintfS("Nextion display - initialize network status\r\n");
//   
//    myPrintfS("Nextion display - initialize Date/Time\r\n");
//#endif

   tempAlarmCheck = flashParameters.tempAlarmCheck;             /* initialize alarm check */
   humidityAlarmCheck = flashParameters.humidityAlarmCheck;     /* initialize alarm check */

    /* shut off buzzer */
    buzzen = 0;
	pwm_set_dt(&pwm_led0, 0, 0); 

    getSensorTemp();      /* display current temp */
//    getSensorHumidity();  /* display current humidity */
//    getSensorPressure();  /* display pressure data */
//    
//    /* reset min/max settings */
    resetMinMax = RESETTEMPMINMAX | RESETHUMIDITYMINMAX | RESETPRESSUREMINMAX | RESETEZOMINMAX;

    /* check if AM2315 1is available */
    if( (sensorTypePresentAll & (1 << SENSORTYPEAM2315C)) != 0)
    {
       sendCommand("page 1");        /* switch to AM22315 page */
       k_sleep(K_MSEC(2000));
       getSensorTemp();      /* display current temp */
       getSensorHumidity();  /* display current humidity */
       recvRetCommandFinished();
    }

#if 0
    /* check if SDP810 is available */
    if((sensorTypePresentAll & (1 << SENSORTYPESDP810)) != 0)
    {
       sendCommand("page PDP");        /* switch to SDP810 page */
       recvRetCommandFinished();
       k_sleep(K_MSEC(2000));
       getSensorTemp();      /* display current temp */
       getSensorPressure();  /* display pressure data */
    }

    /* check if MCP9600 is available */
    if((sensorTypePresentAll & (1 << SENSORTYPEMCP9600)) != 0)
    {
       sendCommand("page 1");        /* switch to MCP9600 page */
       recvRetCommandFinished();
       k_sleep(K_MSEC(2000));
       getSensorTemp();      /* display current temp */
    }

    /* check if MSP300 is available */
    if((sensorTypePresentAll & (1 << SENSORTYPEMSP300)) != 0)
    {
       sendCommand("page 2");        /* switch to MSP300 page */
       recvRetCommandFinished();
       k_sleep(K_MSEC(2000));
       getSensorTemp();      /* display current temp */
       getSensorPressure();  /* display pressure data */
    }
#endif

    myPrintfS("Nextion display - initialize external power status\r\n");
    refreshDisplay();
    setPowerMode(flashParameters.lowPower);          /* set power mode */
    sprintf(str,"lowpower: %s\r\n",powerModes[flashParameters.lowPower]);
    myPrintfS(str);
 
	/* set up application work queue */
////	k_work_q_start(&application_work_q, application_stack_area,  K_THREAD_STACK_SIZEOF(application_stack_area), CONFIG_APPLICATION_WORKQUEUE_PRIORITY);
//    
//    getRTCTime(&time);
//    curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
//    pTime = localtime(&curTime);
//    sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d",pTime->tm_mon+1,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
//    myPrintfS("Enter Stopped CLI. if no key hit within 5 seconds will continue.\r\n");
//    sprintf(str,"%s cmd> ",buf);
//    myPrintfS(str);
//    memset(buf1,0,sizeof(buf1));

    /* start timer and CLI line enter */
//    cliIdleWait = 5 * (1000 / (CLI_IDLE_WORK_DELAY * 2));
    k_timer_init(&cli_command_timer, cli_command_timer_handler, NULL);
    k_timer_start(&cli_command_timer, K_MSEC(CLI_IDLE_WORK_DELAY/2),K_MSEC(CLI_IDLE_WORK_DELAY/2));

    /* start wait in CLI timer */
    k_timer_init(&cli_wait_timer, cli_wait_timer_handler, NULL);
//    k_timer_start(&cli_wait_timer, K_SECONDS(CLI_WAIT_WORK_DELAY),K_SECONDS(0));

    /* start temp alarm timer */
    k_timer_init(&temp_alarm_timer, temp_alarm_timer_handler, NULL);
    k_timer_start(&temp_alarm_timer, K_SECONDS(flashParameters.tempAlarmCheck), K_SECONDS(flashParameters.tempAlarmCheck));

    /* start humidity alarm timer */
    k_timer_init(&humidity_alarm_timer, humidity_alarm_timer_handler, NULL);
    k_timer_start(&humidity_alarm_timer, K_SECONDS(flashParameters.humidityAlarmCheck), K_SECONDS(flashParameters.humidityAlarmCheck));

    /* start pressure alarm timer */
    k_timer_init(&pressure_alarm_timer, pressure_alarm_timer_handler, NULL);
    k_timer_start(&pressure_alarm_timer, K_SECONDS(flashParameters.pressureAlarmCheck), K_SECONDS(flashParameters.pressureAlarmCheck));

    /* start pressure alarm timer */
    k_timer_init(&ezoec_alarm_timer, ezoec_alarm_timer_handler, NULL);
    k_timer_start(&ezoec_alarm_timer, K_SECONDS(flashParameters.ezoecAlarmCheck), K_SECONDS(flashParameters.ezoecAlarmCheck));

    /* start power alarm timer */
    k_timer_init(&power_alarm_timer, power_alarm_timer_handler, NULL);
    k_timer_start(&power_alarm_timer, K_SECONDS(flashParameters.powerAlarmCheck), K_SECONDS(flashParameters.powerAlarmCheck));

    /* start saved JSON packet handler */
    k_timer_init(&json_saved_timer, json_saved_timer_handler, NULL);
    k_timer_start(&json_saved_timer, K_SECONDS(JSON_PACKET_SEND_TIME),K_SECONDS(JSON_PACKET_SEND_TIME));

    /* start saved JSON packet handler */
    k_timer_init(&verify_json_timer, verify_json_timer_handler, NULL);
////    k_timer_start(&verify_json_timer, K_SECONDS(60),K_SECONDS(60));
//#if nextionDisplayAvailable
////    refreshDisplay();
//#endif
    runFlag = false;
    cliChangedCFG = false;
    downlinkChangedCFG = false;
//    resendCFG = false;
}

///*!
// * \fn int jfet_dk_leds_init(void)
// *
// * \brief JFET initialize LED GPIO pins to Output
// *
// *
// * \return None
// */
//static int jfet_dk_leds_init(void)
//{
//	int err = 0;
//
//	for (size_t i = 0; i < ARRAY_SIZE(led_pins) - 1; i++) 
//    {
//		led_devs[i] = device_get_binding(led_pins[i].port);
//		if (!led_devs[i]) 
//        {
//			printk("Cannot bind gpio device %s\r\n",led_pins[i].port);
//			return -ENODEV;
//		}
//
//		err = gpio_pin_configure(led_devs[i], led_pins[i].number, GPIO_OUTPUT);
//		if (err) 
//        {
//			printk("Cannot configure LED gpio %d\r\n",led_pins[i].number);
//			return err;
//		}
//	}
//    return 0;
//}
//


/*!
 * \fn void refreshDisplay(void)
 *
 * \brief refresh all the changing elemnets on the display.
 *
 * will update AC power and battery\n
 * will update alarm indicators on the current page\n
 * will update the next page buttons for the current page\n
 * will update global alarm\n
 * will update buzzer alarm\n
 * will update IMEI and client ID \n
 *
 * \return None\n
 */
void refreshDisplay()
{
    uint8_t i;
    char buf[80],buf1[30];
    uint8_t pageReadCount = 3;
    uint32_t currentPage;
    struct tm  time;
//    time_t curTime;
    time_t upTime;
    struct tm *pTime = &time;

    getRTCTime(&time);
//    printk("RTC   date-time: %02d/%02d/%04d %02d:%02d:%02d (%d UTC)\r\n",time.tm_mon,time.tm_mday,time.tm_year + 1900,time.tm_hour,time.tm_min,time.tm_sec,flashParameters.timeZone);
//    curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
//    curTime = mktime(&time) ; /*convert to local time */
//    pTime = localtime(&curTime);

    if(runFlag)
    {
        /* get current uptime */
        upTime = k_uptime_get();
        date_time_uptime_to_unix_time_ms(&upTime);
        /* get current uptime */
        upTime = k_uptime_get();
        date_time_uptime_to_unix_time_ms(&upTime);
        upTime = (upTime/1000) + (flashParameters.timeZone * 60 * 60);       /* convert to seconds */
        pTime = localtime(&upTime);
        sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d (%d)\r\n",pTime->tm_mon+1,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec, flashParameters.timeZone);
//    printk("Local date-time: %02d/%02d/%04d %02d:%02d:%02d\r\n", pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
//    lastDay = time.tm_mday;
//    sprintf(buf,"%02d/%02d/%04d %02d:%02d:%02d (%d)\r\n", pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 2000,pTime->tm_hour,pTime->tm_min,pTime->tm_sec,flashParameters.timeZone);
        NexText_setText(&text0DateTime,buf);
        NexText_setText(&text1DateTime,buf);
        NexText_setText(&text2DateTime,buf);
    }

    k_sleep(K_MSEC(100));
    /* set up display icons that are page dependent */
    if(networkConnected)
    {
//        sendCommand("vis networkStatus,0");
        NexButton_Set_background_crop_picc(&networkStatus0,2);
        NexButton_Set_background_crop_picc(&networkStatus1,2);
        NexButton_Set_background_crop_picc(&networkStatus2,2);
        recvRetCommandFinished();
    }
    else
    {
//        sendCommand("vis networkStatus,0");
        NexButton_Set_background_crop_picc(&networkStatus0,6);
        NexButton_Set_background_crop_picc(&networkStatus1,6);
        NexButton_Set_background_crop_picc(&networkStatus2,6);
        recvRetCommandFinished();
        recvRetCommandFinished();
    }

//    if(runFlag)
//    {
//        sendCommand("vis updatenow,1");
//        recvRetCommandFinished();
//    }
//    else
//    {
//        sendCommand("vis updatenow,0");
//        recvRetCommandFinished();
//    }

    int test = gpio_pin_get(acpr_dev, ACPR_DEV_PIN);
//    printk("ACPWR %d\r\n",test);
    if(test == 0)
    {
//        myPrintfW("Battery\r\n");
 //      sendCommand("vis acpwrStatus,0");
        NexButton_Set_background_crop_picc(&acpwrStatus0,6);
        NexButton_Set_background_crop_picc(&acpwrStatus1,6);
        NexButton_Set_background_crop_picc(&acpwrStatus2,6);
        recvRetCommandFinished();
//        sendCommand("vis batteryStatus,1");
        NexButton_Set_background_crop_picc(&batteryStatus0,2);
        NexButton_Set_background_crop_picc(&batteryStatus1,2);
        NexButton_Set_background_crop_picc(&batteryStatus2,2);
        recvRetCommandFinished();
    }
    else
    {
//        myPrintfW("AC Power\r\n");
//        sendCommand("vis acpwrStatus,1"); 
        NexButton_Set_background_crop_picc(&acpwrStatus0,2);
        NexButton_Set_background_crop_picc(&acpwrStatus1,2);
        NexButton_Set_background_crop_picc(&acpwrStatus2,2);
        recvRetCommandFinished();
//        sendCommand("vis batteryStatus,0");
        NexButton_Set_background_crop_picc(&batteryStatus0,6);
        NexButton_Set_background_crop_picc(&batteryStatus1,6);
        NexButton_Set_background_crop_picc(&batteryStatus2,6);
        recvRetCommandFinished();
    }

     /* get current page and set up page select icons */
//    uint32_t currentPage;
//    sendCommand("get dp");
//    recvRetNumber(&currentPage);
    while(pageReadCount != 0)
    {
        sendCommand("get dp");
        recvRetNumber(&currentPage);
//        printk("Refresh current page: %d\r\n",currentPage);
        if(currentPage < 3)
            break;
        pageReadCount--;
    }
    if(pageReadCount == 0)
        return;

//#if 1

//    printk("current page R: %d\r\n",currentPage);
    switch(currentPage)
    {
        case 0:
            if( ((sensorTypePresentAll & (1 << SENSORTYPEMCP3425)) != 0) || ((sensorTypePresentAll & (1 << SENSORTYPERTD)) != 0)  || ((sensorTypePresentAll & (1 << SENSORTYPEEZORTD)) != 0))
            {
                /* set MCP3425 alarm sysmbols */
                for(i=0;i < MAX_NUMBER_TEMPERATURE_SENSORS; i++)
                {
                    /* check for MCP3425 present */
                    if(tempSensorsPresent[i] == TEMP_MCP3425)
                    {
                        if(flashParameters.tempAlarms[i].tempAlarm & LOWTEMPENABLE)
                        {
                          NexButton_Set_background_crop_picc(&probe1LowAlarm0,3);
                        }
                        else 
                        {
                          NexButton_Set_background_crop_picc(&probe1LowAlarm0,2);
                        }

                        if(flashParameters.tempAlarms[i].tempAlarm & HIGHTEMPENABLE)
                        {
                          NexButton_Set_background_crop_picc(&probe1HighAlarm0,3);
                        }
                        else 
                        {
                          NexButton_Set_background_crop_picc(&probe1HighAlarm0,2);
                        }
                        if( (tempSensorCurMCP3425 < flashParameters.tempAlarms[i].lowTempCheck/100.0))
                        {
                            NexText_Set_font_color_pco(&probe1,31);
                        }
                        else if( (tempSensorCurMCP3425 > flashParameters.tempAlarms[i].highTempCheck/100.0))
                        {
                            NexText_Set_font_color_pco(&probe1,63488);
                        }
                        else
                        {
                            NexText_Set_font_color_pco(&probe1,2016);
                        }
                    }

                    /* check for RTD present */
                    else if( (tempSensorsPresent[i] == TEMP_RTD) || (tempSensorsPresent[i] == TEMP_EZORTD))
                    {
                        if(flashParameters.tempAlarms[i].tempAlarm & LOWTEMPENABLE)
                        {
                         NexButton_Set_background_crop_picc(&probe2LowAlarm0,3);
                        }
                        else 
                        {
                         NexButton_Set_background_crop_picc(&probe2LowAlarm0,2);
                        }

                        if(flashParameters.tempAlarms[i].tempAlarm & HIGHTEMPENABLE)
                        {
                         NexButton_Set_background_crop_picc(&probe2HighAlarm0,3);
                        }
                        else 
                        {
                          NexButton_Set_background_crop_picc(&probe2HighAlarm0,2);
                        }
                        
                        if( tempSensorsPresent[i] == TEMP_RTD)
                        {
                            if( (tempSensorCurRTD < flashParameters.tempAlarms[i].lowTempCheck/100.0))
                            {
                                NexText_Set_font_color_pco(&probe2,31);
                            }
                            else if( (tempSensorCurRTD > flashParameters.tempAlarms[i].highTempCheck/100.0))
                            {
                                NexText_Set_font_color_pco(&probe2,63488);
                            }
                            else
                            {
                                NexText_Set_font_color_pco(&probe2,2016);
                            }
                        }
                        
                        if( tempSensorsPresent[i] == TEMP_EZORTD)
                        {
                            if( (tempSensorCurEZORTD < flashParameters.tempAlarms[i].lowTempCheck/100.0))
                            {
                                NexText_Set_font_color_pco(&probe2,31);
                            }
                            else if( (tempSensorCurEZORTD > flashParameters.tempAlarms[i].highTempCheck/100.0))
                            {
                                NexText_Set_font_color_pco(&probe2,63488);
                            }
                            else
                            {
                                NexText_Set_font_color_pco(&probe2,2016);
                            }
                        }
                    }
                }
            }
            if((sensorTypePresentAll & (1 << SENSORTYPEAM2315C)) == 0)
            {
                sendCommand("vis nextPageAM,0"); 
                recvRetCommandFinished();
            }
            if(flashParameters.alarmsEnabled)
            {
                NexButton_Set_background_crop_picc(&globalAlarm0,3);
            }
            else
            {
                NexButton_Set_background_crop_picc(&globalAlarm0,2);
            }
            if(flashParameters.tempUnitsCentigrade)
            {
                 NexButton_Set_background_crop_picc(&tempUnits0,2);
            }
            else
            {
                 NexButton_Set_background_crop_picc(&tempUnits0,3);
            }
            if(buzzen)
            {
//               NexButton_Set_background_image_pic(&buzzer0,5);
               NexButton_Set_background_crop_picc(&buzzer0,4);
            }
            else
            {
//                NexButton_Set_background_image_pic(&buzzer0,4);
                NexButton_Set_background_crop_picc(&buzzer0,5);
            }
            if(strlen(flashParameters.clientID) == 15)
            {
                strcpy(buf1,(const char *)&flashParameters.clientID);
                if(strcmp(flashParameters.sensorID,"No ID") == 0)
                  sprintf(buf,"IMEI: %s ID: %s",buf1,"ABCDEF");
                else
                  sprintf(buf,"IMEI: %s ID: %s",buf1,flashParameters.sensorID);
                NexText_setText(&text0IMEI_ID,buf);
            }
            break;
        case 1:
            if( ((sensorTypePresentAll & (1 << SENSORTYPEAM2315C)) != 0)  )
            {
                /* set AM2315 alarm symbols */
                for(i=0;i < MAX_NUMBER_TEMPERATURE_SENSORS; i++)
                {
                    /* check for AM2315 present */
                    if( (tempSensorsPresent[i] == TEMP_AM2315C))
                    {
                        if(flashParameters.tempAlarms[i].tempAlarm & LOWTEMPENABLE)
                        {
                          NexButton_Set_background_crop_picc(&probe1LowAlarm1,3);
                        }
                        else 
                        {
                           NexButton_Set_background_crop_picc(&probe1LowAlarm1,2);
                        }

                        if(flashParameters.tempAlarms[i].tempAlarm & HIGHTEMPENABLE)
                        {
                          NexButton_Set_background_crop_picc(&probe1HighAlarm1,3);
                        }
                        else 
                        {
                          NexButton_Set_background_crop_picc(&probe1HighAlarm1,2);
                        }
                        if( (tempSensorCurAM2315 < flashParameters.tempAlarms[i].lowTempCheck/100.0))
                        {
                            NexText_Set_font_color_pco(&probe3,31);
                        }
                        else if( (tempSensorCurAM2315 > flashParameters.tempAlarms[i].highTempCheck/100.0))
                        {
                            NexText_Set_font_color_pco(&probe3,63488);
                        }
                        else
                        {
                            NexText_Set_font_color_pco(&probe3,2016);
                        }
                    }
				}

                /* set AM2315 high alarm symbols */
                for(i=0;i < MAX_NUMBER_HUMIDITY_SENSORS; i++)
                {
                    /* check for AM2315 present */
                    if(  (humiditySensorsPresent[i] == HUMIDITY_AM2315C) )
                    {
                        if(flashParameters.humidityAlarms[i].humidityAlarm & LOWHUMIDITYENABLE)
                        {
                           NexButton_Set_background_crop_picc(&probe2LowAlarm1,3);
                        }
                        else 
                        {
                           NexButton_Set_background_crop_picc(&probe2LowAlarm1,2);
                        }

                        if(flashParameters.humidityAlarms[i].humidityAlarm & HIGHHUMIDITYENABLE)
                        {
                           NexButton_Set_background_crop_picc(&probe2HighAlarm1,3);
                        }
                        else 
                        {
                            NexButton_Set_background_crop_picc(&probe2HighAlarm1,2);
                        }
                        if( (humiditySensorCurAM2315 < flashParameters.humidityAlarms[i].lowHumidityCheck/100.0))
                        {
                            NexText_Set_font_color_pco(&humid3,31);
                        }
                        else if( (humiditySensorCurAM2315 > flashParameters.humidityAlarms[i].highHumidityCheck/100.0))
                        {
                            NexText_Set_font_color_pco(&humid3,63488);
                        }
                        else
                        {
                            NexText_Set_font_color_pco(&humid3,2016);
                        }
                    }
				}
			}
            if( ((sensorTypePresentAll & (1 << SENSORTYPEAM2315C)) == 0) )
            {
                sendCommand("vis nextPageAM,0"); 
                recvRetCommandFinished();
            }
            if(flashParameters.alarmsEnabled)
            {
                NexButton_Set_background_crop_picc(&globalAlarm1,3);
            }
            else
            {
                NexButton_Set_background_crop_picc(&globalAlarm1,2);
            }
            if(flashParameters.tempUnitsCentigrade)
            {
                 NexButton_Set_background_crop_picc(&tempUnits1,2);
            }
            else
            {
                 NexButton_Set_background_crop_picc(&tempUnits1,3);
            }
            if(buzzen)
            {
//               NexButton_Set_background_image_pic(&buzzer0,5);
               NexButton_Set_background_crop_picc(&buzzer1,4);
            }
            else
            {
//                NexButton_Set_background_image_pic(&buzzer0,4);
                NexButton_Set_background_crop_picc(&buzzer1,5);
            }
            if(strlen(flashParameters.clientID) == 15)
            {
                strcpy(buf1,(const char *)&flashParameters.clientID);
                sprintf(buf,"IMEI: %s ID: %s",buf1,flashParameters.sensorID);
                NexText_setText(&text1IMEI_ID,buf);
            }
            break;

//        case 2:
////            NexText_Set_font_color_pco(&probe4,2016);
//            if((sensorTypePresentAll & (1 << SENSORTYPESDP810)) != 0)
//            {
//#if 1
//                /* set AM2315 alarm sysmbols */
//                for(i=0;i < MAX_NUMBER_TEMPERATURE_SENSORS; i++)
//                {
//                    /* check for SDP810 present */
//                    if(tempSensorsPresent[i] == TEMP_SDP810)
//                    {
//#if 1
//                        if(flashParameters.tempAlarms[i].tempAlarm & LOWTEMPENABLE)
//                        {
//                          NexButton_Set_background_crop_picc(&probe1LowAlarm2,3);
//                        }
//                        else 
//                        {
//                           NexButton_Set_background_crop_picc(&probe1LowAlarm2,2);
//                        }
//
//                        if(flashParameters.tempAlarms[i].tempAlarm & HIGHTEMPENABLE)
//                        {
//                          NexButton_Set_background_crop_picc(&probe1HighAlarm2,3);
//                        }
//                        else 
//                        {
//                          NexButton_Set_background_crop_picc(&probe1HighAlarm2,2);
//                        }
//#endif
////                        printk("%3.2f %3.2f %3.2f\r\n",tempSensorCurSDP810, flashParameters.tempAlarms[i].lowTempCheck/100.0,flashParameters.tempAlarms[i].highTempCheck/100.0);
//                        if( (tempSensorCurSDP810 < flashParameters.tempAlarms[i].lowTempCheck/100.0))
//                        {
//                            NexText_Set_font_color_pco(&probe4,31);
//                        }
//                        else if( (tempSensorCurSDP810 > flashParameters.tempAlarms[i].highTempCheck/100.0))
//                        {
//                            NexText_Set_font_color_pco(&probe4,63488);
//                        }
//                        else
//                        {
//                            NexText_Set_font_color_pco(&probe4,2016);
//                        }
//                    }
//                }
//#endif
//#if 1
//                /* set AM2315 alarm sysmbols */
//                for(i=0;i < MAX_NUMBER_PRESSURE_SENSORS; i++)
//                {
//                    /* check for SDP810 present */
//                    if(pressureSensorsPresent[i] == PRESSURE_SDP810)
//                    {
//#if 1
//                        if(flashParameters.pressureAlarms[i].pressureAlarm & LOWPRESSUREENABLE)
//                        {
//                           NexButton_Set_background_crop_picc(&probe2LowAlarm2,3);
//                        }
//                        else 
//                        {
//                           NexButton_Set_background_crop_picc(&probe2LowAlarm2,2);
//                        }
//
//                        if(flashParameters.pressureAlarms[i].pressureAlarm & HIGHPRESSUREENABLE)
//                        {
//                           NexButton_Set_background_crop_picc(&probe2HighAlarm2,3);
//                        }
//                        else 
//                        {
//                            NexButton_Set_background_crop_picc(&probe2HighAlarm2,2);
//                        }
//#endif
//                        if( (pressureSensorCurSDP810 < flashParameters.pressureAlarms[i].lowPressureCheck/100.0))
//                        {
//                            NexText_Set_font_color_pco(&pressure4,31);
//                        }
//                        else if( (pressureSensorCurSDP810 > flashParameters.pressureAlarms[i].highPressureCheck/100.0))
//                        {
//                            NexText_Set_font_color_pco(&pressure4,63488);
//                        }
//                        else
//                        {
//                            NexText_Set_font_color_pco(&pressure4,2016);
//                        }
//                    }
//                }
//#endif
//            }
//            
//            if((sensorTypePresentAll & (1 << SENSORTYPEMSP300)) != 0)
//            {
//#if 1
//                /* set AM2315 alarm sysmbols */
//                for(i=0;i < MAX_NUMBER_TEMPERATURE_SENSORS; i++)
//                {
//                    /* check for MSP300 present */
//                    if(tempSensorsPresent[i] == TEMP_MSP300)
//                    {
//#if 1
//                        if(flashParameters.tempAlarms[i].tempAlarm & LOWTEMPENABLE)
//                        {
//                          NexButton_Set_background_crop_picc(&probe1LowAlarm2,3);
//                        }
//                        else 
//                        {
//                           NexButton_Set_background_crop_picc(&probe1LowAlarm2,2);
//                        }
//
//                        if(flashParameters.tempAlarms[i].tempAlarm & HIGHTEMPENABLE)
//                        {
//                          NexButton_Set_background_crop_picc(&probe1HighAlarm2,3);
//                        }
//                        else 
//                        {
//                          NexButton_Set_background_crop_picc(&probe1HighAlarm2,2);
//                        }
//#endif
////                        printk("%3.2f %3.2f %3.2f\r\n",tempSensorCurMSP300, flashParameters.tempAlarms[i].lowTempCheck/100.0,flashParameters.tempAlarms[i].highTempCheck/100.0);
//                        if( (tempSensorCurMSP300 < flashParameters.tempAlarms[i].lowTempCheck/100.0))
//                        {
//                            NexText_Set_font_color_pco(&probe4,31);
//                        }
//                        else if( (tempSensorCurMSP300 > flashParameters.tempAlarms[i].highTempCheck/100.0))
//                        {
//                            NexText_Set_font_color_pco(&probe4,63488);
//                        }
//                        else
//                        {
//                            NexText_Set_font_color_pco(&probe4,2016);
//                        }
//                    }
//                }
//#endif
//#if 1
//                /* set AM2315 alarm sysmbols */
//                for(i=0;i < MAX_NUMBER_PRESSURE_SENSORS; i++)
//                {
//                    /* check for MSP300 present */
//                    if(pressureSensorsPresent[i] == PRESSURE_MSP300)
//                    {
//#if 1
//                        if(flashParameters.pressureAlarms[i].pressureAlarm & LOWPRESSUREENABLE)
//                        {
//                           NexButton_Set_background_crop_picc(&probe2LowAlarm2,3);
//                        }
//                        else 
//                        {
//                           NexButton_Set_background_crop_picc(&probe2LowAlarm2,2);
//                        }
//
//                        if(flashParameters.pressureAlarms[i].pressureAlarm & HIGHPRESSUREENABLE)
//                        {
//                           NexButton_Set_background_crop_picc(&probe2HighAlarm2,3);
//                        }
//                        else 
//                        {
//                            NexButton_Set_background_crop_picc(&probe2HighAlarm2,2);
//                        }
//#endif
//                        if( (pressureSensorCurMSP300 < flashParameters.pressureAlarms[i].lowPressureCheck/100.0))
//                        {
//                            NexText_Set_font_color_pco(&pressure4,31);
//                        }
//                        else if( (pressureSensorCurMSP300 > flashParameters.pressureAlarms[i].highPressureCheck/100.0))
//                        {
//                            NexText_Set_font_color_pco(&pressure4,63488);
//                        }
//                        else
//                        {
//                            NexText_Set_font_color_pco(&pressure4,2016);
//                        }
//                    }
//                }
//#endif
//            }
//
////            if((sensorTypePresentAll & (1 << SENSORTYPESDP810)) == 0)
////            {
////                sendCommand("vis nextPageAM,0"); 
////                recvRetCommandFinished();
////            }
//            if(flashParameters.alarmsEnabled)
//            {
//                NexButton_Set_background_crop_picc(&globalAlarm2,3);
//            }
//            else
//            {
//                NexButton_Set_background_crop_picc(&globalAlarm2,2);
//            }
//            if(flashParameters.tempUnitsCentigrade)
//            {
//                 NexButton_Set_background_crop_picc(&tempUnits2,2);
//            }
//            else
//            {
//                 NexButton_Set_background_crop_picc(&tempUnits2,3);
//            }
//            if(buzzen)
//            {
////               NexButton_Set_background_image_pic(&buzzer0,5);
//               NexButton_Set_background_crop_picc(&buzzer2,4);
//            }
//            else
//            {
////                NexButton_Set_background_image_pic(&buzzer0,4);
//                NexButton_Set_background_crop_picc(&buzzer2,5);
//            }
//            if(strlen(flashParameters.clientID) == 15)
//            {
//                strcpy(buf1,(const char *)&flashParameters.clientID);
//                sprintf(buf,"IMEI: %s ID: %s",buf1,flashParameters.sensorID);
//                NexText_setText(&text2IMEI_ID,buf);
//            }
//            break;
    }
//#endif
}

 /*!
 * \fn void uart0_cb(const struct device *x, void *user_data)
 *
 * \brief UART (CLI console) call back function
 *
 * called on a UART interrupt
 *
 * \param x pointer to a device structure
 * \param user_data user data to be passed to callback
 *
 * \return none
 */
void uart0_cb(const struct device *dev, void *user_data)
{
    uint8_t i;
	int data_length = 0;
    uint8_t temp_buf[16];

	uart_irq_update(dev);
	if (uart_irq_rx_ready(dev)) 
    {
		data_length = uart_fifo_read(dev, temp_buf, sizeof(temp_buf));
		if(data_length > 0)
		{
			for(i=0; i < data_length; i++)
			{
				uartData.data[fifoPtr++] = temp_buf[i];
				uartData.len = fifoPtr;
				if(temp_buf[i] == '\r')
				{
					k_fifo_put(&fifo_uart_rx_data, &uartData);
					fifoPtr = 0;
				}
			}
		}
	}
}

 #if 0
 /*!
 * \fn void uart2_cb(const struct device *x, void *user_data)
 *
 * \brief UART2 (Nextion display) call back function
 *
 * called on a UART interrupt
 *
 * \param x pointer to a device structure
 * \param user_data user data to be passed to callback
 *
 * \return none
 */
void uart2_cb(const struct device *dev, void *user_data)
{
   uint8_t i;
	int data_length = 0;
    uint8_t temp_buf[16];

	uart_irq_update(dev);
	if (uart_irq_rx_ready(dev)) 
    {
		data_length = uart_fifo_read(dev, temp_buf, sizeof(temp_buf));
		if(data_length > 0)
		{
			for(i=0; i < data_length; i++)
			{
				uart2Data.data[fifoPtr2++] = temp_buf[i];
				uart2Data.len = fifoPtr2;
				if(temp_buf[i] == '\r')
				{
					k_fifo_put(&fifo_uart2_rx_data, &uart2Data);
					fifoPtr2 = 0;
				}
			}
		}
	}
}
#endif

 /*!
 * \fn void uart2_cb(const struct device *x, void *user_data)
 *
 * \brief UART2 call back function
 *
 * called on a UART interrupt
 *
 * \param x pointer to a device structure
 * \param user_data user data to be passed to callback
 *
 * \return none
 */
void uart2_cb(const struct device *x, void *user_data)
{
    uint8_t i;
	int data_length = 0;
    uint8_t temp_buf[16];
    static int count=0;

	uart_irq_update(x);
    count++;
//    dk_set_led(LED2,count & 0x01);
//    printk("uart_cb called\r\n");
	if (uart_irq_rx_ready(x)) 
    {
		data_length = uart_fifo_read(x, temp_buf, sizeof(temp_buf));
//		uart_buf[data_length] = 0;
	}
//    printk("%d\r\n",data_length);
    for(i=0;i<data_length;i++)
    {
        uart2_buf[num_chars_seen++] = temp_buf[i];
        if( (uart2_buf[0] == 0x65) || (uart2_buf[0] == 0x67) || (uart2_buf[0] == 0x68))
          touchSeen = 1;
        if(temp_buf[i] == 0xff)
        {
          num_end_seen++;
          if(num_end_seen == 3)
          {
              display_line_seen = true;
          }
        }
//      printk("%d %02X %d %d %d\r\n",i, temp_buf[i], num_chars_seen,num_end_seen,display_line_seen);
    }
}

// /*!
// * \fn void  checkDayChange(void)
// *
// * \brief checks if the day has changed
// *
// * checks if day has changed and if so restarts the
// * data timer to send a JSON packet immediately
// * with the message "daily Update"
// *
// *
// * \return none
// */
//void  checkDayChange(void)
//{
//    struct tm time;
//    time_t curTime;
//    struct tm *pTime = &time;
// 
//    if(correctTimeSeen == 0)     /* check if correct time/date seen yet */
//    {
//        return;
//    }
//
//    getRTCTime(&time);
////    printk("RTCx   date-time: %02d/%02d/%04d %02d:%02d:%02d (%d UTC)\r\n",time.tm_mon+1,time.tm_mday,time.tm_year + 1900,time.tm_hour,time.tm_min,time.tm_sec,flashParameters.timeZone);
//    curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
//    pTime = localtime(&curTime);
////    printk("Local date-time: %02d/%02d/%04d %02d:%02d:%02d\r\n", pTime->tm_mon+1,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
//    
//
//    /* check if day changed so need to reset min/max sensor values for new day */
//    if( (lastDay != pTime->tm_mday) && (dailyUpdate == 0) )
////    if(lastDay != pTime->tm_mday)
//    {
//        printk("RTC   date-time: %02d/%02d/%04d %02d:%02d:%02d (%d UTC)\r\n",time.tm_mon,time.tm_mday,time.tm_year + 1900,time.tm_hour,time.tm_min,time.tm_sec,flashParameters.timeZone);
//        printk("Local date-time: %02d/%02d/%04d %02d:%02d:%02d TZC: %d TZO:%d\r\n", pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec, flashParameters.timeZone, originalTimeZone );
//        printk("last day: %d current day: %d\r\n",lastDay,pTime->tm_mday);
//        printk("Day changed from %d to %d so reset Min/Max\r\n",lastDay,pTime->tm_mday);
//        lastDay = pTime->tm_mday;
//        dailyUpdate = 1;
//        if(strlen(message) == 0)
//            strcpy(message,"Daily Update");
//        k_timer_start(&data_sample_timer, K_SECONDS(0), K_SECONDS(flashParameters.dutyCycle));
//    }
//}

 /*!
 * \fn void  checkHourChange(void)
 *
 * \brief checks if the hour has changed
 *
 * checks if hour has changed and if so 
 * check for 0/8/16 and restarts the
 * data timer to send a JSON packet immediately
 * with the message "daily Update"
 *
 *
 * \return none
 */
void  checkHourChange(void)
{
    struct tm time;
    time_t curTime;
    struct tm *pTime = &time;
    char str[90];
//	int trys;
//	int res1, res2;
 
//    myPrintfW("checkHourChange 1\r\n");
//   if(correctTimeSeen == 0)     /* check if correct time/date seen yet */
//    {
//        return;
//    }
//    myPrintfW("checkHourChange 2\r\n");

    if(!runFlag) 
    {
        return;
    }       /* check if network connected */
//        time_NTP_server_get();
//        printk("Downlink NTP date %s(UTC)\r\n",sntpTime);

//    for (trys=0; trys< 4; trys++)
//	{
//		res1 = modem_info_params_init(&modem_param1);
//    	res2 = modem_info_params_get(&modem_param1);
		modem_info_params_init(&modem_param1);
    	modem_info_params_get(&modem_param1);
//		if ( (res1 + res2) == 0)	/* chacek if both passed */
//			break;
//		printk("trys: %d res1: %d res2 %d\r\n",trys,res1,res2);
//	}
//    printk("Modem data_time: %s %d\r\n",modem_param1.network.date_time.value_string, strlen(modem_param1.network.date_time.value_string));
    strcpy(modemTime,modem_param1.network.date_time.value_string);
    getRTCTime(&time);
//    printk("RTC   date-time: %02d/%02d/%04d %02d:%02d:%02d (%d UTC)\r\n",time.tm_mon+1,time.tm_mday,time.tm_year + 1900,time.tm_hour,time.tm_min,time.tm_sec,flashParameters.timeZone);
//    curTime = mktime(&time) + (flashParameters.timeZone * 60 * 60); /*convert to local time */
    curTime = mktime(&time); /*convert to local time */
    pTime = localtime(&curTime);
 //   printk("Local date-time: %02d/%02d/%04d %02d:%02d:%02d\r\n", pTime->tm_mon+1,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
    
    if( lastHour != pTime->tm_hour) 
    {
        sprintf(str,"hour changed from %d to %d %d\r\n",lastHour,pTime->tm_hour,dailyUpdate);
        myPrintfW(str);
    }

    /* check if day changed so need to reset min/max sensor values for new day */
    if( (lastHour != pTime->tm_hour) && (dailyUpdate == 0) )
    {
        if( (pTime->tm_hour == 0) || (pTime->tm_hour == 8) || (pTime->tm_hour == 16) )
        {
            sprintf(str,"RTC   date-time: %02d/%02d/%04d %02d:%02d:%02d (%d UTC)\r\n",time.tm_mon,time.tm_mday,time.tm_year + 1900,time.tm_hour,time.tm_min,time.tm_sec,flashParameters.timeZone);
            myPrintfW(str);
            sprintf(str,"Local date-time: %02d/%02d/%04d %02d:%02d:%02d TZC: %d TZO:%d\r\n", pTime->tm_mon,pTime->tm_mday,pTime->tm_year + 1900,pTime->tm_hour,pTime->tm_min,pTime->tm_sec, flashParameters.timeZone, originalTimeZone );
            myPrintfW(str);
            sprintf(str,"last Hour: %d current Hour: %d\r\n",lastHour,pTime->tm_hour);
            myPrintfW(str);
            sprintf(str,"Hour changed from %d to %d so reset Min/Max\r\n",lastHour,pTime->tm_hour);
            myPrintfW(str);
            dailyUpdate = 1;
            if(strlen(message) == 0)
                strcpy(message,"Daily Update");
            k_timer_start(&data_sample_timer, K_SECONDS(0), K_SECONDS(flashParameters.dutyCycle));
            setDateFromModem();     /* on daily update resync the RTC */

        }
        lastHour = pTime->tm_hour;
    }
}

/**
 * \fn void myPrintf(char *str)
 *
 * \brief prints a string to the UART
 *
 * @param str ASCII string to print
 *
 * \return HAL_OK if passed<p>
 *
 */
void myPrintf(char *str)
{
    printk("%s",str);
  	return ;
}

/*
 * Black \033[0;30m.
 * Red \033[0;31m.
 * Green \033[0;32m.
 * Yellow \033[0;33m.
 * Blue \033[0;34m.
 * Purple \033[0;35m.
 * Cyan \033[0;36m.
 * White \033[0;37m.
*/

/* yellow (Warning)*/
void myPrintfW(char *str)
{
    printk("%s","\33[1;33m");
    printk("%s",str);
    printk("%s","\33[0m");
    return;
}

/* Green (Information)*/
void myPrintfI(char *str)
{
    printk("%s","\33[1;32m");
    printk("%s",str);
    printk("%s","\33[0m");
    return;
}

/* CYAN (Status)*/
void myPrintfS(char *str)
{
    printk("%s","\33[1;36m");
    printk("%s",str);
    printk("%s","\33[0m");
    return;
}

/* Red (Error)*/
void myPrintfE(char *str)
{
    printk("%s","\33[1;31m");
    printk("%s",str);
    printk("%s","\33[0m");
    return;
}

void myPrintnextIon(char *str)
{

	for(int i=0;i<strlen(str);i++)
	{
		uart_poll_out(uart2_dev,str[i]);
	}
	return;
}
